from core import models
from core.translation import _t, HTMLColorToRGBA

def canvas_size(image,new_size,align_hor,align_ver,back_colour,opacity,
        old_size=None):
    #because of layer support photo size can be different from image layer size
    if old_size is None:
        old_size    = image.size
    #check if image size has changed
    if old_size == new_size:
        return image
    #displacement
    dx              = new_size[0]-old_size[0]
    dy              = new_size[1]-old_size[1]
    #horizontal alignment
    if align_hor == 'left':     x   = 0
    elif align_hor == 'center': x   = dx/2
    else:                       x   = dx
    #vertical alignment
    if align_ver == 'top':      y   = 0
    elif align_ver == 'middle': y   = dy/2
    else:                       y   = dy
    #mode
    if image.mode == 'RGBA':
        back_colour = HTMLColorToRGBA(back_colour,opacity)
        print back_colour
    new_canvas      = Image.new(image.mode,new_size,back_colour)
    new_canvas.paste(image,(x,y))
    
    return new_canvas

class Action(models.Action):
    """Resize an image"""
    
    label       = _t('Canvas size')
    author      = 'Stani'
    email       = 'spe.stani.be@gmail.com'
    version     = '0.1'
    tags        = [_t('image')]
    __doc__     = _t('Scale the canvas without resizing the image')
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Width')]                  = models.PixelField('100%')
        fields[_t('Height')]                 = models.PixelField('100%')
        fields[_t('Resolution')]             = models.DpiField('<%s>'%_t('dpi'))
        fields[_t('Align Horizontal')]       = models.AlignHorizontalField('center')
        fields[_t('Align Vertical')]         = models.AlignVerticalField('middle')
        fields[_t('Background Colour')]      = models.ColourField('#FFFFFF')
        fields[_t('Opacity')]                = models.SliderField(100,0,100)
        
        super(Action,self).__init__(fields)
        
    def import_modules(self):
        global Image
        import Image
        
    def apply(self,photo,setting,cache):
        #get info
        info                = photo.get_info()
        #dpi
        dpi                 = self.get_field_value('Resolution',info)
        #size
        info                = photo.get_info()
        x0, y0              = info['new_size']
        x1                  = self.get_field_size('Width',info,x0,dpi)
        y1                  = self.get_field_size('Height',info,y0,dpi)
        #parameters
        parameters          = {
            'old_size'      : (x0,y0),
            'new_size'      : (x1,y1),
            'align_hor'     : self.get_field_value('Align Horizontal', info),
            'align_ver'     : self.get_field_value('Align Vertical', info),
            'back_colour'   : self.get_field_value('Background Colour', info),
            'opacity'       : int((self.get_field_value('Opacity',info)/100.0)*255),
        }

        #manipulate layer
        return self.apply_to_current_layer_image(photo,canvas_size,**parameters) 
    
    description = """
This action offers four different resampling filters.
- NEAREST. Pick the nearest pixel from the input image. Ignore all other input
  pixels.
- BILINEAR. Use linear interpolation over a 2x2 environment in the input image.
  Note that in the current version of PIL, this filter uses a fixed input environment
  when downsampling.
- BICUBIC. Use cubic interpolation over a 4x4 environment in the input image.
  Note that in the current version of PIL, this filter uses a fixed input environment
  when downsampling.
- ANTIALIAS. Calculate the output pixel value using a
  high-quality resampling filter (a truncated sinc) on all pixels that may contribute
  to the output value. In the current version of PIL, this filter can only be used
  with the resize and thumbnail methods.
Note that the ANTIALIAS filter is the only filter that behaves
properly when downsampling (that is, when converting a large image to a small one). The
BILINEAR and BICUBIC filters use a fixed input environment, and are best used for
scale-preserving geometric transforms and upsamping.
"""

    icon = \
'x\xda\x01\x86\x0by\xf4\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\x00\
\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\x08\
\x08|\x08d\x88\x00\x00\x0b=IDATh\x81\xed\x99\xd9\x8f]G^\xc7?Uu\xce\xb9K\xf7\
\xed\xdd\xeen\xdb\x1d\xef[{!q&A\x80B\xd0\xcc\xa0\xd1H\x08$\x16\xb1\x83\x84\
\xf8\x0fxBB\xca\x13O\x88\x17\x02\xe2\x8dy@B\x1a? \xcd \x01\x12\x0c\x96\x82D\
\x04\xa3\t\x89\x93vl\xb7\x9d\xde\xdc\xcb\xed\xbe[\xf7]\xcf9U\xf5\xe3\xe1\xf6\
\xb5\xbb\xdd\xd7\x1d{4\xc9\x8c\x84\x7f\xa5\xd29\xfa\xdd:u\xbf\xdf\xdf\xf9\
\xfej;\xf0\xd2^\xda\xffoS?n\x00{m\xe1\xd6\xb7F2\xd1\xd0\xf90\x9b\xbf4\x90\rn\
\xfc\xc9\x9f\xbf\xfb\xdd\xbf\xfd\x87\xef\xde:\xec\x99\xe0\xcb\x02w\x88\xa9\
\xb5\xf7nN\x10\x04\xb3a\x14\xcez\xd4\re\xcc\x15\xeb9{nf\xb2\n\xfcd\x11X\xb9\
\xf5\xf7\'2\xf9\xc2\x19Q\xfa\x9a\x0e\xc2\x1b W\x15\x9c\xc7\xfbQ\xf1\x16\xed,\
Is\x1b\t4\xb3\xa7g~\t\xf8K\xa0\xf9\xac\xfe\xbe0\x02\xef\xbc\xf3\x8e\xfe\xe3o\
^?ftx6\x08\x83\x1bJ\xd45\x85zMi\xf3\x8a6fL\xbcCl\x82Oc\xf0\x16%\x9eP)\xac\
\xf3\xc4I\x8cV\x11gf&O\x8d\x8e\x0e\x9c\xaaV\x9bs_(\x81[\xb7\xde\t\xce\xe5\
\xaeOEAnV)sU\x1b\xbe\xa2D]R\xf83"nXl\x8a8\x8b\xf7\x0e\xad\r*\x08\xd1\xde\x83\
w$IL\xbd\xd3\xa6\xd2n\xb1\xde\xdc!\xca\x18\xae\x8f\x1d\xc3YK!\x97\x19?91q\
\xf2GJ\xe0\xe6\xcd\x9b\xe6\xed\x13\x99\xd3A>\x7f\xce\xb9\xf4u\x1dD\xd7D\xfc%\
\xbc?+\xde\rJ\xda"\xedX\xbc\xb5(\x04\xa3@+\x05\x02i\x9a\xd2\x8c;T\xdb-*i\x87\
\x8dv\x83\xb2M\xd8\xc1\x11\x1b\xb0\xa1f\xc8\x05\\f\x12\xe5<C\x85\xbc~u\xf6\
\xcc\xec\x87\xf3K\xff\xfcC\x11X\xb8\xf5\xadl\xbe0y\\\xbc\xbb\x12d\xf3W\xf0\
\xfeU\xad\xf55\xf1nF\x89\x1d4x\\{\x07gS\x10\x8f\x064\nq\x0e%B#\x8e\xa9\xc7\
\x1d\x8a\xed\x06[\x9d6e\xdb\xa1&)i\xa8\xb1F\x11\x0eF\x18\x93#\x0c\r\x19\xad1\
\xc6\xe0%\xa5\xee\x13\x86\x9c\xc2\x18\xc7\xc5S\xc7\xaf\x02\x06p\x87\x12X\xfe\
\xaf\x9b\xb9\x0c\x99\xe3:\x1b]Q\xca\\U\xda|\x05\xfc\x05\xa58\x1b\x18\x9d\xc1\
\xb5\x11\xe5p\x89\xc5\xa7\x0e\xbc\xa0\x04"\xadIDh\xc61\xe5V\x9bR\xbbI1nRJ;4\
\xb5\xd0\xc4\xa1"\x83\xca\x19\xc2(K`\xf2]\xb0\x81Fk\x8d\x0e4&0h\xa3\xd1F\x13\
\xfb\x94\xedNB\xc1\x85X\x9b0{\xfa\xd8%`\x08\xa8\xf6%\xf0\xc6\xec\xec\xd4\xbf\
}\xfb\xdd\xbf\xf2\xe8\xd7\x95b\x1a\xefs"\x1e\xf1\tx\x87V V\xd0\xa2\x10\xe7I\
\xe3\x94\xedV\x8b\x8d\x9dmJ\xed\x16\xa5\xb4C]9\xb6}J\x12\x82\x04\x06\x9d\x0b\
\x08\x0b\x19\x02\xa3\x196\x06\xad\xd5.P\x8d6]\xb0J+\x94\xeaNC\x82\x80\x80\
\x17\x8f\x03\xca\xa4\x9c\xf0\x86$M919vf|pp\xb2\xdch\xf4\'\x90\xc9\x9a\xe3\
\xa4\xed\xdfh5[\x04F\x11jCh\x0c\xceYR\xeb(\xb7\x9aT;-6\x9au\xb6\xe2\x16%\x1b\
\xd3R\x9e$P\x10\x1aL>\xc0\x98\x900\xc8\x91\xd1\nm4\xc6\xec\x89j\xa0\xbb9\xa0\
\xbby \x08"\x82 x\xf1 <!Awf\xadi\x8f\xc7\x91\xc6\xc2\xd1\xe1\x81\xb1\x99c\
\xe3\xa7\xcb\xf7\x1bw\xfb\x12\xb8\xb7\xf2p\xb5T\xae\xac\x8f\x0c\rN\xb7\xdb\
\x1dj\xae\xc1\xa7[\xebl\xb6[\x94mL[CG\x0b&\x1b\xa2\xf2\x9a(\xca\x11\x1aCv\
\x17l\x17\xb0A\x87]I(\xa3P(Pt\x81\x8a\xe0\xbb\xe1ED\xf6\x81\x05\x1e\xfbz~\
\x05\xd4\x94\xc5\xe9\x0cib\xc9F\x91\xb9|\xfa\xd8\x85\x0f\xef/\xfdK_\x02[[\
\xad\xca\xc2\xa3\xb5\x07o^\xbb4M\x18p\xaf\xb3\xc5\x7f$\x9b\x0c\x15\n\x04\xc1\
\x00\x911\xe4w_\xb9\xeeE6\xe8\x02W\xea\xa0\x0cDv#\xbb\x07T?\xa0\xcf" @GASY\
\xc6\xbc\xc2ha\xf6\xd4\xf4\xf5\xdd\x97\xf3\xa4q\x8f\x00\x90\xdc\xbe\xfbp\xee\
g~\xea\xd2[mk\x99\xce\r116D>;\x88\x0e\x14\xc6\x18\x94Qh\xad\xf7\xff\xd1!2\
\xd8G\xa8\x8f\xefYm{\xd6\xc6RW\x9e1\xa7\x88\xe3\x94\xd9\xd3\xd3\x17\x81\x02\
\xb0\xd3\x8f\x00\x0f\x96V\xef \x1e\xefSF"\xc3\xf8\xd0 >\x93\xe9R\xee\xc9@\
\xfcsG\xf10\xff\xf3\xb4\xf5JQR\t\'}D\'N895~.\x8a\xa2\xa9$I\x0e\x10\xd0\x00\
\xdf\xff\xe4\xfe\xedz\xbd)\x1e\xcf\xa0\t\xc89!u\x0e\xe7\x1c\xce;\xbc\xf8/\
\xb5\x8a\xf7\x94\xb5\x03%\xd84e\xac\x90\x9f\xb8\xf6\xca\xe4\xc9\xa7\xc1?&\
\xb0\xbaZ^h4[\x9b\x88\xa0\x9cc\x9c\x0c\x89\xa4\x8f\xa5\xf2\xb8c\x04\x8f\x7f\
\xfc6\x9e\xcb\xff4\xb8\xe7\xe8C!T\xb5\xa5\xed-\x88\'\x1b\x1933=v\xf1\x99\x04\
\xd6\xab\xd5\xf5\xbb\x9f-=\x8c\x8c\x06\xf1\x8c\x8a\xc1\xef\x89\xfc\xbe?z\xc1\
\xfb\xcfo{\xb0\r^h\xa8\xee\xf2B\x8b\xc7;\xc7\x8d\x8b3\xb3=\xbc{\xad7\x13\xa7\
k\x9b\x959\r?\x9b\xa4\x961\x1f\xa0\xbd\xc7)\x0f\xcf1\xf4=\xed\x7f\xf1\xfc\
\xe0\x80?UP\xd3\x96c"x\xe78w\xfc\xc8U \x0f4\x0e\xbc\x01\x80\xff\xfc`\xee\xb6\
Q\xe0\x9c\xa3 \x86\xc8\x0bN\xfc\x8bK\x06\xbf?\xf1y2Zu}\xdd*\x80g\xf7~O\x1fV\
\x1cN\x1c\x89x*\xda\xa1\x11\xac\xb5\x9c?6qz(\x93\x99\xea+!\x80\xb5R\xf5^\x9c\
$\xe2\xbc\'+BA\x0c\xa9\xb8\xe7\x94L\x7f\xa9\xf8=\x80\x1f\x03?\xa4\xbf\xd4[\n\
:K\xa8\x02\xacX6\x95\x05\x14\xde;\xc6\x87rSS\xe3\xc3\x07\x12\xf91\x81\x0f\
\xee=\xb8\xbfU\xd9\xae(\x11\x8c\x17\xc6\x9c\xc6\xc9\xf3\x8e@\x07\x81\xfa>ypX\
M\xbd%k"~\xe7\xc4W\xf9\xdd\xc9\x9fc&\x1ce\x956\xa9\x1240\x90\xcf\x06\xd7/L\
\x1eH\xe4\xc7\x04j\xb5\xce\xe6f\xa9\xf2\xd0\x18\x85x\xc7\xb8\x048o\x9fLX\xfb\
$s\xb8\x0cz\xe5EF\xaaD,o\x15.p\x94\x88c\xd1(\x7f0\xf1\xd3\xdc(\x9c\xa1\xa5\
\x1d\xda;\xa2L\x9eW\x8e\x8c^\xe6\xa9\x83\x88\xbdY\xdd\xbe\xfd`\xe9\xd3(\x8ap\
"\x14\x12!\x10\xd5\xcd\x03\x11Dx.\x19\xf4\xae{\xef?O\x82\x89X.\x9bQ\xae\xa6Y\
j\xe5G\xb4\xda5\xa2 \xcb\xdb\xa3W8:y\x01\t3$I\xcc\x1b\xe7gf\x81\xc1\xbd\x04\
\xf6mh\x1e,o~(\xce\xfe\xa1\xf5\x8eA\xa7\xc8x\xa1\xa5\x05\xfd\x82\xeb\x99\xa7\
\xfd\x87\xaf}\x84\x00\xc5\x8d\xb4\x80\x94V\xb1\xb9\x01\x9cMH\xc3\x16\xe1\xc0\
\x10\x99\xf1\x19\x94\x0e\xa0\xb1\xc9\xf4\xd8\xe09\xba{\x83z\xaf\x0f\xb3\x8f\
\x8dR\xc3\xbf\xf9\x8bo\xfe^\x92\xa6\x18\xefY\nSv\xb4G\x89<\x06\xd1\x9b\xdc\
\xf6\x02\xe8\x01\xdb\xd7\xa6W\xfa\xb5}\xca\xef5,\xef\x94\x89W\xcaLE!a\x18\
\x806\xb84\xc6\x8a\xa0\xf3#\xa0\r#Y\x93_\xde\xd8\xfc\xe8\xceb\xf1\xe3\xbe\
\x04\xb2\x04\xfc\xfa\xd7_\xff\xfd@\x933\x0eJ\x81\xe3\x91N\x08P\x9f\x0fV\x84\
\xa7\xcb\xb3\xda\xf6\xfcOf^\xd8\xd6\x96\xf7\xe6\x17\xa9UkL\xa519\x1c&\x93%\
\x08B\x10\x87\xce\x8d\x10\x15F\xcc\xd7\xae\x9d\xf8\xc6\xe4`\xe0\xfe\xfd\x07\
\xf3\x1f\x02\xe9>\x02\x95V\xab\xf3+o]\xff\xb5\xc9\xd1\xc2\xb4K-\xcd@\xf8\xcc\
$\xe8>Q=P\xfa\xf9\xfa\xf8\xbdx\xacOIl\x8c(\xc8E9\x9cwh\r\xcdV\x87\xb9\xe66\
\x0b\xb6I\xbeQg\xb0\xb5\x83\xc6c\x82\x10%\x1e\xa7#\xc2\xe1\xa9\xe8k\xd7O}\
\xfd\xee\xfc\xfc\xf7\xe7\x966\xef>\xbd\xa9o/\xacn}r\xfd\xcc\xf4k\x1dg\x19\
\xf7\x06\xbc\xc3+\xf5\x98\xc2>\xad\xf7[\x1a\xf7\xd3=B\xeaR\x9c8\xf2Q\x9e\xe9\
\xc1W89t\x92\xe9\x81\x13H\x0c\xdfY\xfc\x0e&\x10.\x9d=\x03\x83m\xf2bh\x12R,m\
\xb1S\xd9dlj\x86\x81\xc9S\xe8\xdc :7\xc8\xb7\xff{\xe9\xe3\x7f\xfa\x9f\xbb\
\x8f\x80\xf0\xc0\xa9\xc4\x07w\x97?\xf9\xd5\xb7_%u\x96\x81T\x91\x0f\xa1e\x1c\
\xfa\x10\xb0{\xfdO\x13\xf0\xe2I\xc5qf\xf8,\xaf\x1e{\x8d\x89\xe0\x08:Q4w*lo\
\xad\x10\xb7\xb6\xf8\xe5`\x8a\xd1T\x18\xb4)*\x17\xd0\xa8\xd6\x88\xd3\x06&\
\x1f\xa1\\J\xf5\xd1<\xbeYa\xf8\x95\xcbl\x94\xca\x9d?\xfb\x8b\xbf\xf9\xbbv\
\xdb\x96\x019@`~\xa5\xf8Q\x92$Xg)\xf8\x90\x01\xeb\xd9\xd1\x8a\xe0\xf3\x08\
\xecj\xda\x89\xc7!(\xa5\tL@\xde\x0c0mr|c\xfcM\xdc\xc26I\xe3}\x82\xb4J!mrT,\
\xa1s\xe0\xba\xcf*\ra\x14\x92=2\x8c\xf3\x9ev\xe2\xa8n\xd7\xfd|\xa9SZ\xfe\xc1\
\xfb;\xbfpm\xf5\xf4_\x7fo\xe1\x1f?[Y\x7f\x0f(\x02\xf6\x00\x81;\xcbk\x0b\xf5f\
\xbb\xa6`\x84\xd41\x11\x19V$E\xa3\xf6I\xc6K\x17\xacW\x00\x8a\xc0\x84\xe4M\
\x8e\x11\x93\xe7\x88\xceqTe8b5#\x89#\xdfL\xb0k\xffJ\xa5R&o\x14\xc6(\xa2@\x93\
\xcd\x04d\xf2\x11hE\'\xf1\x14+\x8d\xf4\xfe\x9djma\xb3\xba\xf6\xe9Bqma\xb3\
\xb6|o\xa5\xb4Tkt6R\xefw\xce\x7f\xef\xce\xa9\xf9r\xe7\x0e\xf0\x08hA\x9f\x83\
\xad\xb5R}\xf5Q\xb1\xb2<39:\x82wL8M\xe2\x12P\xa6;\x89)E\xa8\x03\xf2a\x9eQ\
\x93g\\\xe7\x98"\xcb\x84U\x0c\xa7\x9e|#FwZ\xf8\xb8\x84\xb3\t"\x0e\xab\x15&\
\xd0\x1c\x1f\x1b\xc4\x8a\xd0\x89S6\xb6\xeb\xe9z\xa5^\xbd\xb7R*\xce\xaf\x95\
\x96\xee,\x97\x96W6\xb7W\xd6\xab\xadu\xa0\x02\xd4vA6w\xaf\xc9|\xb9\xf3>`w\
\x7f\xa7/\x01\xa0\xf9pmk\xee\xdc\xf1\xf1\xeb>v\x8cu,C\xd9\x0c\x13\xd9\x11&t\
\x96I\xc90\xe1\x0c\xc3\xb1#\x1b\xc7\xe8N\x13I\xcax\x9b\xe0\xc5\x11+0F\x81\
\xd2\x88\x12\x1aqJ\xb5\xdeN\x977\xab\xa5\x87k\x95\xe2\xbd\xd5\xf2\xe2\xfc\
\xa3\xf2\xca\xd2\xd6\xcer\xb9\xde\xda\x04\xcat\xf7\xba\x8d]\xa0-\xa0\r\xc4@\
\x02\xa4\x80\xef\x83\xf3\x99\x04\xf8\xe8\xc1\xea\xedo\xbeq\xf1\xb7;b\x19\x8b\
\x03\xfe\xc8\x1d!\xbf#\x84q\x1bI\xebx\x97\xe0\xc5\xef\x9eL\x80\x89\x14\xa91\
\xb4b\xcff\xb5\xdeY,V\xab\x0f\xd7+\xabs\x0b\xc5\x95\xc5\xad\xed\xa5\xc5\xf5\
\xcaz\xb5\xd5Y\xf7\xbe/\xd8\xcenMv\xa3\xfbL\xb0\xcfM\xe0\xde\xa3\xe2\x9cBH\
\xd2\x94(0d\xab\x9b\xa0\x15*\x0c0\xd9\x10\xaf24;\tk[\xd5xiuc\xeb\xe3\xc5\x8d\
\xb5\xe5bmen\xb9\xb8R\xac6W\xb7\xb6[\xebt\x8f\x02{2\xe8E\xb5W\xd3\x1f\x06\
\xecs\x13\x98[(\xde\xaf7\xdb\xe9P>\n5Pk\xb4)\xd6\x1a\xad\xcf\xd6*\xa5\xc5\
\x8d\xca\xc6\xc7K\x1bK\x0b\xeb\xb5\xe5\xc5bu\xa5\xdeN\xb6\x80\x12\xdd\xa8\
\xf6\xf4\xfa\xb4\x0c,}\xcet~\x14\xf6\xacod\x03\x7f\xfa[?\xff\xae\xd1\xe6\xd2\
\xff>X[^X//\xad\x94vV\xeb\xed\xa4H7\xaau\x9e\xc8\xa0\x17\xd5/\x1cl?{\x16\x81\
\x008\x01\x9c\xa2\x0blod{\x9a\xed\xc9\xe0K\x03\xdb\xcf\x0e\xfbJ\x19\x01\x19\
\xba\xe7\xf2\xc9\xee\xf5\xc7\n\xf6EM\xd1\xdd\xf0\xfcD}\x8a}i/\xed\xa5\xed\
\xb7\xff\x03\x8c\xcc"\xd3\xc0\xe57j\x00\x00\x00\x00IEND\xaeB`\x82\xce[\xadC'