import os
import Image
from core import ct, models
from core.translation import _t

JPG = ['jpg','jpeg','jpe']

FILENAME        = '<%s>'%_t('filename')
FOLDER          = '<%s>'%_t('folder')
FOLDER_PHATCH   = '<%s>_%s'%(_t('folder'),_t('copy'))
TYPE            = '<%s>'%_t('type')
DPI             = '<%s>'%_t('dpi')

class Action(models.Action):
    """Defined variables: <filename> <type> <folder> <width> <height>"""
    
    label       = _t('Save')
    author      = 'Stani'
    email       = 'spe.stani.be@gmail.com'
    version     = '0.1'
    tags        = [_t('save')]
    __doc__     = _t('Save an image')
    save        = True
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Filename')]     = models.FileNameField(FILENAME)
        fields[_t('As')]           = models.ImageWriteTypeField(TYPE)
        fields[_t('In')]           = models.FilePathField(FOLDER_PHATCH)
        fields[_t('Quality')]      = models.SliderField(60,1,100)
        fields[_t('Resolution')]   = models.DpiField(DPI)
        
        super(Action,self).__init__(fields)
        
    def apply(self,photo,setting,cache):
        #get info
        info                    = photo.get_info()
        #get file values
        folder, filename, typ   = self.get_filename(info)
        if setting('overwrite_existing_images') or not os.path.exists(filename):
            #get other values
            dpi     = self.get_field_value('Resolution',info)
            #filename
            if setting('create_missing_folders'):
                models.ensure_path(folder)
            #construct options
            options = {'dpi' : (dpi,dpi)}
            if typ.lower() in JPG or \
                    (typ.lower() == '<type>' and self.type in JPG):
                quality             = self.get_field_value('Quality',info)
                options['quality']  = quality
            #save
            photo.save(filename, **options)
        return photo
    
    def get_filename(self,info):
        folder      = self.get_field_value('In',info)
        filename    = self.get_field_value('Filename',info)
        typ         = self.get_field_value('As',info)
        return folder, os.path.join(folder,'%s.%s'%(filename,typ)), typ
        
    def is_done(self,photo):
        """Method used for resuming when a batch was interrupted. C
        Check if this image has been done already."""
        #get info
        info                        = photo.get_info_original()
        #check if there are not forbidden tags (new_*)
        try:
            folder, filename, typ   = self.get_filename(info)
        except KeyError:
            return False
        #check if file exists
        if not os.path.exists(filename):
            return False
        #check if file is valid
        try:
            Image.open(filename)
            return True
        except:
            return False

    def is_overwrite_existing_images_forced(self):
        return (self.get_field_string('In') == FOLDER) and\
            (self.get_field_string('Filename') == FILENAME) and\
            (self.get_field_string('As') == TYPE)
    
    description = _t("""
""")

    icon = \
'x\xda\x01\xd6\x08)\xf7\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\x00\
\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\x08\
\x08|\x08d\x88\x00\x00\x08\x8dIDATh\x81\xedYK\x8c\x1cG\x19\xfe\xfe\xbf\xaa{\
\xbag\xe7\xb1\xbb~\xec\xda^\xd6\xf6\xda$^\x1c\xf0\xda\x1c \xe1`\x11^!\x04!\
\x8c\x88\xc2\x058\xa0H<$\x8e9E\x1c\xa2HpD" 8pBr\x84,\x82y\x1e\x08\xa0\x00\
\x11\x1cL\xec\x04$$\xb0l\xe7%\x88\x9c\xf5zwggvg\xba\xfe\x9fC?\xa6\xa7\xa7gwm\
\xb3\'\\R\xa9\xaak\xfe\xea\xfa\xbe\xffUU=\xc0\xddr\xb7\xfc\x7f\x17\xda\xae\
\xe0\xc2\xc9Gp\xe9\xe2/\xf0\xf4\xb7\x9e\xa1\x85w\x1f\xaf\xd4\x1b\xb5\xc0\xaf\
\xf8\x81g\xbd\x90\x88,\x00&\x80\x01bU\xa5\xf8\xdd\x04b0\x94\x0c\x11\x08PQ\
\x85\xa8B\x01\xd5xy\xcd/\xe3\x00\xf4Dd\xa3\xbb\xb1\xb1\xf6\xd6\xf5\xc5\xe5s\
\xcf\xff1z\xcf\xbdsx\xe2k_\xb8}\x02\x7f\xbd\xf8\x0fD\xceMU\xc3\xe0L\x18\x84\
\x0fx\x9e\xbf\xdbz\xb6\xc6\xc4uf\xae\x81\xc8\'\xc0\x10\x81\x012\x008}7Qq\x19\
\x85\xea\xd0\x12\xe9\x88S\xd5\r\xe7\xdc\xfa\xc6\xc6\xfa\x8d\xcez\xe7\xcdN\
\xbbs~m\xads\xceX\xd3~\xe0\xfd\'\x86&\xda\xad\xc0\xbf\xf8\xe2%<\xf7\x93\xe7\
\xe9\xd1G\x1f\xfaz\xbdV{"\x0c\xab\\\xa9\x04`\xe6x\xe524%c\xdb)\x14\xb3\x85\
\xaa\xc2\x18\x86a\x02\x01\x0f\x8b\xc8\xc4\x89\x85\xc3\xdf~\xe1\x85\x97p\xfa\
\xf4\xa9\x819\xbc\xe5K\r\xe1c\x0f}`\x8a\x88>."\x1cE\x11\xa2\xa8\x87^\xaf\x87\
(\x8a\xe0\x9c\x83\x88\x0c\xd4\xdb!\xa0\xaa\xd9|\xe7\x1c\xa2(B\xaf\x17\x01*\
\x811t\xe6O\x7f\xf8{\x93x\xf8\xbd[Z\xc09\x07Uy\'\xa0G|\xbf\x82 \x08Q\xa9T2m\
\xedD\x89\x15\xa0P\x118\xd7\x83\x88\xcc(\xa4\xae\x82\xe5\xa2\xec\x96\x16`\
\x12\x88\xb8\xfd\x9e\xe7\x85\xd6\x9a\xccuv\xbe\x10@\x80\xe7y\xb0\xd6\xf8N\\M\
T\x86\xf1m\xf5\x9aC\xb33\xf0=o\xde\xf7=K\xc4;\xaay \xd6~f\x01\x15\x00\nkMUU\
\xc7\x9d\xb8!\xf9M\t\x9c\xff\xf5\xefp`f\x1a\xcc|\xc0\x18\x0b"\xca\xeaN\x80V\
\x15\xa8\nD\x1cD\\\xfc\x0c\x855\x1c\x1a\xc3\xbb\x8c\x19\x86\xbbi\x0c\xb02\
\xce\xfe\xf8\x97cA\x10\xcc2\xf3\xff\x14|>\xd0\xfb\xe0\xe3\xedAT\xa0\xe2\x92\
\x84 `\xc3\x15\xc3\xd44\xc6\x0cc\xdcl\x11\x179\xf8\x9eW\xb7\xd6\xce\x1af\xa4\
$\xee\x14x^\xe3\xb1\xb6\xa3L\xeb.\xed\'\xd6P\x11\x186LD\x81\xef[\x9c\xfb\xe9\
o\x06\xde\xb7\xa9\x05\xea\xf518\xe7v\xf9\xbe\xd7\xa4;\xb0\xc0\xa0\xb6\xa5\
\xe02\xb1\xd6\x8b\xe3"\x0e*\x0e\xaa\nk\r\x8c5\x13\x07g\xa7\xf1\xda\xeb\xd7\
\xb7O`\xac\x1ab\xa3\xdb;\xec\xfb\xfex\x0c\xfeV2P\x7f\xc7\xcd\x023\x05\x89a\
\xc0E\x19\x91t\x7fq`\xf6\xe0Y\xbbwjj\x1f\xa0\xaf\x0c\xac2\x12\xd1\x8f\xce\
\x9eG\xa3Q\x83\xef\xdb}\x869\xdc\xae\xf6\xcb]$v\x13\'.\xae.\x828\x97\xd4\xa4\
_\x90Qq\x10\x8d\x03\x99\x88`\x0cOS\xf5$\x15\x11\x8f$\xc0l1\x7fl\x0ea\x18\xcc\
\x19k\x90\xa6\xd02\x12\x83\xa0c\xed\xb9\x01\x80Q\xce\xd7\xa3\xcc\xd73\x7f\
\x97\xa8O$\'\xaf\xc9\xaeND`\xe6\xe6G\x1e<\xc5ZH\xa5#\t\x18\x02\xee?\xfd9C\
\xc4G\x99M)\xf8\xed\x04d\xaaQ)\xd3\xba\x8b\xad\x91\xc9\xe4\xe6\xa6i\x14\xd0\
\xd4\x02\x95\xf7\x9ez\x97Y\xefF\x03\x18F\xc6@7r\xf8\xe2\xe7\xcf\x8c\x05\x15\
\x7fo\xea\xff\xf9\xc3V\xdcn/ c\x9f\xef?\xa3\xc4\xf7\xb3\xf9\xd9\xc14\x8d9\
\x053\xc1Z[\x9f\x9d\xd9\x17:\'\xddm\x11\x00\x08\x8dZm\xb7\xe7\xd9w\x10\xfa\
\xda\xcf\x0ek\xb7\x10\x90C`1,\x93\xae\t"\x10\x18\xcc\n\xe68\x95"\xb6\xc0Du,\
\xa81\r\x9e\x87F\x12\xa8\x86\x01\x88h\x92\x89\xc7\x13\x87\x81$\xfe\xd7\x073J\
\xeb\xe5 \x87\x08\x0eh;\x01O\x04\x02A\x15`\x16\x88\x12\x98\x18\xcc<^\xf1l\
\xadx\xd0\x1dI\xe0\xe0\xc1\xfdX\xba\xb9r\xc8\xf7\xbd\xaa\xa8\x80\x13\x1fG\t\
\xc0mi}\x94\x8b$\x803\x02I_\x01\xb0\x08\xd4\xc5\xf7\x04\xc3&`\xe6\x8a\x93\
\xc1\x03])\x81\xef\xfc\xf09L\x8c\xd7\xb1\xbc\xbcz\x98\rUT\xd23J\x7f\xdb/\xf5\
\xe3\x91Z\x1f\xd6v\x0c\x983\x8d\xe7\xb5\x1f\x13\x89O\xc2\xca\n\x10\xc1Zc*\
\xbe\x172\x0f&\x92\xd2,\xd4m\xb5pdn\x16\xd508\xc0Dq>\x96\xc1,3\x94\x06e0\r:\
\x97\xcf$\x88\x93\x00\x1bpR\xe3>\x83\xc9\x94\x8c\xc7-\x99\xa4\x0f\x821\xc6\
\xf3<oW\xb3\xd9\xd8\xda\x02\x86\t_\xfa\xca7\x02\xdf\xf7\xee\x89\xaf\xe2\x02Q\
\x07r\x18\xa9\xf5~\x1c \xd1vN\xcb%.\x92i\x9c\x08@n\x8fI\xad\x00@8>\x0b\xc5Al\
*\xd6\xda\xe9\xb1\xb1\xea\xd6\x04\xacg0;3U5\x96\xa7\xc4\t\x9c\x8b\x92\x85\
\x86\x83t\x00x\xfc\x1d"\x03\x92<\r\xf7\xb3\x94L\xc0&\xf2H\xc7\x15`fc\x8c\x99\
\x0c\x82`k\x02a\xe0azjr\xaa\xe2\xd9=\xbd^\x17\xbd^/q\x87t\xc3\xea\x07d\x82{\
\x00\x08\xa1\xe0\xe3\xd4\x07\x1aW\xc9\xc9\xf5\x03y\xd0*\x04\xe7\xe2\xfbq|\
\xadU0#\x0c\x03\x1fO>\xf5\x03<\xf5\xe4\xe3\xa3\t\xf8\x9e\x07\xe7\xdc>6\xb4+>\
\x8b(\xd2M8\xde\xd6M\xb9;\x0c\x04#\x0f\x01\x1ap\x93Q\xf2\xb1.\xa0\n\x10)\xa0\
.\xcbJ\xd6\x9a\xc6\xdc\xdc!\x8c7\xfb\xb0K\t\xcc\xcf\x1f\xc1\xe5\xcb\xd7\x0e\
\x00\xecw:]\xb4\xdb\xed\x8c@\xbfP\xe62\xfd\xa1\xcc\x16\xc8\x90 \xdf\xa5\x82\
\\n,\xff\xb3&1\x90\xa4\xee\xb1j\x15\xbe\xef\x81\xd9L}\xe6\xb1\xaf\xda\x0f?x\
\x7fv\x9e\x18"\xf0\xcc\xf7\xbe\x8f\x93\x0b\xc7\xb1\xba\xdaY\xb8v\xedu\xfb\
\xf2+/cee\x15\xddn\x17"\xc3\x97\xea\xc2\x97\xb5\xed\x95\xfc\x14\x02\x8c\xb1\
\xf0=?\x19V\xf4\xbce\xb0\x8d\xa0]\x1f\xd65\xd0l\xd4\xb0\xb0p\x12\x95J\xb0\
\xff\xd3\x9f\xfc\x90?V\x1b\x1bM`br?\xce>\xfb\xf3\xda\xe4\xe4\xf8\x89\xabW.\
\xa3\xd3\x89p\xe1\xc2\x05\xbc\xfa\xea\x15\xac\xad\xaen\x02\xf76?f\x81Po4ql\
\xfe>\x9c>\xfdQ\xb4\xcc\x1b\x18\x9b_\xc3\xfe\xa9}\xf8\xcf\xbf\x97 WwC\xd7<\\\
\xbe\xfc/\xcc\x1d\xb9gbbr\x97\xcd\xef\x05C\x04Z\xad\x0e\x9csc\xa2:]\xadV\xd1\
lFh\xb7n\xc0r|C\x1b\x80\x19\x7f\xe4\xcc\xfa}*\xe9Mf\x98\xa3\x16\x06\x15\n\
\xe76ps\xe9-4\xea!\xcc\xb8b\xfcH\x0bA\xe35\xec\x9eX\x83\xc5\x02\x82\xf6,\xae\
\xbf\xfd&\x16\x17\x97xi\xa5M\x9e\xed\xdf\x8d\x87\x088\'X\\\xbc\xb1BDW\x1a5\
\xff\xf8\xa9\x93\'0\x7f\xec(\xa2(B\xf6U`H\xd9EX\x83\xa0\xcb\x87\xfa?\xb8\xc8\
!\x08\xab\x98\x98\xd8\x83\xb6\xee\xc1\xb5\xf6?\xb1Z\xbd\n\xbf{\x183\x93G!A\
\x05\xab\xab\x01\xd6\xda\xeb7n.-\xf7\xc2\xb02\x9a\xc0\x97\x1f\x7f\x8c\x01t\
\x9f\xfe\xe6w\xcf\xab\x8e\x7f\xb0\xf2\xf6b\xad\xd9\xa8\xc3\xf3\x92I#newr\xd5\
W\x00*\x8a\xe5\xe5\x1b\x00\x0c\x1a\x8b\x9f\xc2\xc6\x1b\x8b\xa8`/V\xb5\x87\
\xe5\x95E\xac\xb46Z\xadV\xe7\xd9c\xf7\xdd\xdb\xfe\xdb\x85K\xa5\xebrB\xc8\x00\
\xb0\xf3\xc7\x17\xaa\x9fx\xe4\xb3\x0f\xd7\xeb\xf5\xf7\x11\xc1W\x15\x12\xd1\
\xe4\xd3\xb9Rr\x04\xa6\xf4Szr\xb4&h|\x94T\x80\xa0JI\x1b\xff\x96\xc8A\x91\x8d\
\'JQ"R&\x12b\x12&\xabLF@*`(\x94\xbakk\xad?\xff\xfe\xb7\xbf\xfa\xd9\xc5\x97\
\xfe\xb2\x04 Jxg\x04L\x02\xde\xe6\xfa&G*_\xd3O\xe7\\\xe8\x17\xdb\xcd~K\xb7\
\xe1\xbc\x11\\\xaeF\x85\xb6\x07`\x1d@7i;\x006\x00h\n\xd2\xe4\x00\x9b\\\x1dEj\
\xbb\x00\xf3u\xc0c\x00\xc8\x88\xaa9\xb9\xb4\xa4\xf3\xf32\x0e\x80\xe4c ?1_%\
\x01%\x050\xa9\xbc\xe4\xc6\xd2\xbf\\\xf2\xc0\x8b\xbf\x17K\xd9z\xf9\x9aZ"\xb5\
F:\x0e \xd6j\xfa@\x89@\xda\x16\x89\xa5\xcc\x05\xb1%F\x01-j\xbdh\x81\xad\x08\
\xe4I\xa4\xe0{\x88\xdd\'\xad\xbd\x14c1\x88\xf3\x81\\\xe6V\xc5~\xd1\x9dn\x05x\
^9)\xe8\xb4\xcd\xc7B\xaf@"}\xce\x16)\x96\x14\x90\x19\xd1nV7\xf3\xfd\xed\x02\
\x17\x94\x07qJj`g\xd9JC\xc5\xacR\x06xT\xf0\x8ez\x7f\x99\xbb\x14\t8\x0c\xfa\
\xfb\xc8}\xf2V\xf7\x9f2\ro\xa6\xf5b\xf6)#PF\xe6\x96\x00\xedT)\xbe\xfb\xf6N{w\
\xcb\xdd\xb2\xb3\xe5\xbfdr\xfcH\xc0\xebQd\x00\x00\x00\x00IEND\xaeB`\x82\xaez\
C\xd3'