from core import models
from core.translation import _t

METHODS = [_t('Rotate 90'), _t('Rotate 180'), _t('Rotate 270'),
            _t('Flip Left Right'),_t('Flip Top Bottom')]

class Action(models.Action):
    """"""
    
    label       = _t('Transpose')
    author      = 'Stani'
    email       = 'spe.stani.be@gmail.com'
    version     = '0.1'
    tags        = [_t('Image')]
    __doc__     = _t('Flip or rotate 90 degrees')
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Method')]   = models.ChoiceField(METHODS[0],METHODS)
        
        super(Action,self).__init__(fields)
        
    def apply(self,photo,setting,cache):
        import Image
        #get info
        info        = photo.get_info()
        #get file values
        method      = self.get_field_value('Method',info)
        #get layer
        layer       = photo.get_layer()
        #manipulate layer
        method      = getattr(Image,method.upper().replace(' ','_'))
        layer.image = layer.image.transpose(method)
        #save layer
        photo.set_layer(layer)
        photo.set_size(layer.image.size)
        return photo
    
    description = _t("""Description""")

    icon = \
'x\xda\x01>\x16\xc1\xe9\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x000\x00\
\x00\x000\x08\x06\x00\x00\x00W\x02\xf9\x87\x00\x00\x00\x04sBIT\x08\x08\x08\
\x08|\x08d\x88\x00\x00\x15\xf5IDATh\x81\xbd\x9ai\x90\x9c\xc7}\xde\x7f\xdd\
\xfd\xdes\xed\xce\xeeb\x17\xbb\xb8/\x02$\x01\x02\xe6%\x92\x12\x18\x8a\xa2-\
\x99Q\xd9\x8e\x9d\xb8R\xb2b\xf9\x83\xcba\x89\x8c\x15I\xfe\x10\xcb\xe5T\xa5$\
\x95\xa8\xc4.W\xd9\xa6+\xf9\xeaH\x14\x1d3vL\xd1<M\x11\x00\x01\x12"\x88\x83\
\xb8\x81\xc5\xb5\xc0\xeebwvfw\xe7~\x8f\xee\xce\x87\x19,)3\xf2\x119\xe9\xaa\
\xaewj\xb7\xde\xe9\xe7\xf9\x9fOw\x8f\xe0\'\x1c\xff\xf9\xf7~O\x0e\x0e\x0c\xf8\
\xbe\xe7\x15\x03?\x18\x94R\x96\xa5\x94\x83R\xaa<\xd6\xba\x16k\x8d6\xdd4M\x1b\
\x99\xd6\xb5$I\xaa\xedv{i\xa1Vm]\x9a\xbc\x98j\xad\xedw\xbe\xf3\x9d\xff\xeb\
\xf5\xc5?\xf6\x85\xdf\xfe\x0f_\xe3\xf2\xe5\xab<\xf2\xc8^\x7f\xd5\xc8\xc8p\
\x18F\x1b\x1c\xd7\xd9*\xa5\xd8"\x84\\o\xad\x185P\xb2\xc6\x86\xc6X\x07\xac\
\x05\x91\x08l\xcbZ\xb3ht6\x9d\xa4\xc9\x958\x8e/v\xba\xf1\xa5V\xbb}\xfd\xea\
\xd5\xcb\xcb\xdf~\xfai\xed\xe7\\\xe2V\xfa\xff\x86\xc0\xb7\xff\xcb\xb7\xb9|\
\xe5\x12w\xed\xdc\x13L\xac\x9eX\x97\xcbE{\\\xd7\xfb\x98\x94\xea.m\xc4\x86nJ\
\xb9\x9b\x10\xb6cT\'\xb1"M5Z\x1b\x01\xa0\x94\xb2\xbe+\xf1=k\x02\xc7\xa6\x9e\
\xa3\x9b\x92l\xde\xe8\xf4B\x92t\xdfk\xb7;\x87\xeb\x8d\xc6\xe9\xa3\xc7\x8e\
\xcd\xff\xb7\xff\xfa\x8c\xdeq\xc7\xed\x9c;s\xf6\x9f\x8e\xc0\xef\xff\xfe\x1f\
\xe08\x8e\xb3n\xdd\xba\xf1b\xb1\xf8\x90\xef\xfb\x9f\x02uo7e\xddrK\xe4\x17\
\xeaBV\xeb\x82\xc5&\xb6\xd1\x11tbk\x93Lc\x8c\xc1\x02J\x08\\G\x08\xdf5\xe4C\
\xcb`\x0e1\\\xb4\x0c\xe5u\x96\xf3u\r\x1b_\x88\xbb\xdd\x03\xadV\xf3\xf5j\xb5z\
\xfc\xcd\x1f\xbc\xb1\xb4\xb4\xb4h\xdf\xdc\xbf\xef\'#\xf0\xcc\x1f\xff\x11\xcf\
\xfd\xd9\xb3|\xe97\xbf\x92\x1f\x1eY\xb5\'\x0c\xa3_\xb0\x88G\x9b\x1d\xbbq~\
\x89\xdc\xf5\x8a\x1035e\x17\x1a\x82V\xc7\xda$E\xa4\xdab\x8c\xc5Z\x83\xb5\x16\
\x8b\x85\xfeg\xacA\n\x83\xebX\x91\xf3-#E+\xd6\x8e\x08\xd6\x8e\x90\rFY\xcd\
\x9a\xce\xc9v\xbb\xfd\xfd\xe5\xe5\xa5\x97\xe6\xe6\xe6.GQ.]\\\xac\xf1\xf5\xaf\
\xff\xa7\x1f\x8bQ\xfd\xb8\x7f<\xfb\xddg9z\xf4\x18\xbf\xf1\x1bO\x0c\xadY\xb3\
\xe6\xd3\x03\x03\xe5\x7f\x8b\xf4~v\xb6f\xd7\x9d\x9e\x12\xfe\x89\xab\x0e\x17g\
\x147\x17\x05\x8d\xb6 N\xad\xc8\xb4\xc1\x1a#\xb0F\x08iE\xe0\t\x11\xfaR\xf8.\
\xc2w\x10\x8eBdF\x8b814;\x9aj\xdd\x88\xe9\xaa\xa6\xb2d\xa4\x15**\xe4\xbc5a\
\xe0\xec\xf0=w(\x0c\xc2j\xab\xdd\xaa\xdcq\xc7\xael\xf3\xe6\r\x1c8p\xe0\x1fN\
\xe0\xc5\xef\xbf\xc8\xbb\x87\xdf\xe3\x97\xfe\xe5\xbf\x18[\xb3f\xcd/\x96\x06\
\xcaO\xb4\x13\xf5\xb1s\xd7\xd2\xe2;\xe7\xac8=%\x99[\x84N\x8c\xd0\xc6\x88\x1e\
h-\xc0\x08zV\x17\xf9P\x8a\x07v\x96\xc4\xde=\x03b\xd7\xd6\xbc\xd8\xb9%\'\xb6\
\xae\rD\x9a$ba)\x16Z\x1ba\x8c!N4\x8b\x8d\x8c\xd9jJ\xa3\x83\x93\x8f\xdc\x81b\
\xe4n\x0e|g\xcc\xf7\xbc\xda\xf4\x8d\xa9\x9b_\xfb\x9d\xaf\xa5q\xa7\xcb[\x07\
\x0f\xfe\xfd\x04^y\xe5e\x8e\x1c\xfb!?\xf3\xe9O\x8f\xae\x99\x98\xf8\xe5Bi\xe0\
\xd7\x17[\xf2\xce\xc3g\xda\xde[\'\x13\xae\xdc\xb4\xa2\xd5\xb1B\x1b+\xb0\x06\
\x8b\xe9\x87\x88\xc1\x82\xc0\xf6\x80\r\x95\x1c>\xf3\xd00\x8f\xdc]f\xeb\xfa\
\x88-kB&\x86\x1df\xe6\x97\x98\xbc\xde$\xd5\x16\xac\xc5Z\x8d1\x96N\xacEe)e\
\xb1n\x84\xef9\xe1@\xc1]\x17\xf8j\xd4\xf3\xdc\xca+\xaf\xbc2\xfd\xbb\xff\xf1w\
\xd34\xd3\x1c8\xb0\xffG\xf0:\x7f\x9b\xc0\xf5\xeb7\xf8\xd8}\x0f\r\x8e\x8e\xae\
\xfa\xb9\\\xbe\xf8k\x0buy\xfb\xfe\x13u\xf5\xf6\xc9\xd8\xce/\x1a\x91\xe9\x1e`\
\xb0\xe2V\\[k\xfb\xb3G\xc4\x98\xde\xd3U\x16G\x81\x14`\x00)5\x98\x04kR\x8cq\
\x80\x0f\xde\xb1\xc6\xd2\x8d\r\x17o\xa44\xda\xa9h\'a\xe1\x8e\xb5\xc1?\xcb\
\x17\xc8\xc6FM\xf7\x9b\xdf\xfc\xd6[\xdf\xfc\xc6\xd7\x93\xbf\xd3\x03\xcf>\xfb\
=\xa2(\x8a\xc6\xc7\xc7\x1f+\x95\x06\x9fXn\xab\x9d\xfbO4\xdd\x83\'Zv\xae\x96\
\t\xad\r`\xfa\xc0{\x04V@\xf7\x9fBX\x04\x96bNq\xd7\xb6"\xe3\xc3\x1e\xc6\x18\
\xb4\xceh\xb5cN]\\\xe0\xc2T\x874\x03\xac\xc1X\x0bVc\xfb\xa4\xb51\xd4[)\x8b\
\x8d\x8c\\\xe8x\xc3%o\xccwe\xe8(yi\xc7\x8e;*\xf7\xdcs\x8f}\xe7\xf0;\x1f\xf5\
\xc03\xcf<\xc3\xb5k\xd7\xd4=\xf7\xdc\xb3\xcbs\x83\xcf-5\xb2\xbb\x8eN\xc6\xde\
;\'c;\xbf\x98\tm\x0c\xc2\x1aa\xb1X\xdb\x0f\x1fk\x11\x18\\WP\x88\x14\xe5\xa2G\
\xb9\xe8`\x8d\xa6\x9bh\x8cNI\xd2\xa4\x07Tk\xd2$\xc1h\x8d5\x1ac\xfa\x0b[\xfdA\
\xc5\xb2=\xefjc\x98YH9|Z\x13\xf9\xc5\xe2\xa6\xb1\xe0\xb1b\xb1tc\xcd\x9a\x89\
\xea\x93O}\xf1\xfaC\x0f~\x9c\x83\x87\xde\xfa\x80\xc0\xd3O\x7f\x8b\x1d;\xee\
\xa0V\xab\xae\n\x82\xf0\xb3\x9dX?ti\xaa\x19\x1e>\x037k\xbaoy+\xfa\x0b\t\xd3\
\x0b!\x02\x17\xc6\x86<\xb6\xad\r\xd9\xb26`\xac\xec2\x90W\xd4\xea\x1d~\xf8~\
\x85,\xed\x10w}\xc0`\x8c&MS\xb4\xd6\x18\xab{Q\x88\xc5r\xab\xc4\xda\x15O\xde\
\xca\xa9\xebsm\x8e\x9cC\x0c\xe4J\xab\xca\xf9\xf0\xf1\x81\x81\xc13_\xfb\xed\
\xdf\xf9\x0b\xcf\xf3\xda?B`\xf5\xea\tN\xbc\x7f\xcc\xbb\xe3\xf6\x9d\xf7\x0b\
\xe9<6\xb3\x90\x0c\x1d\xbb\xe80]\x91d\x99\xc1Z#\xb0VX\x0c\xd6\x18\x94\x84\
\xa1\x92\xe2\xae\xad\x11\xf7\xec(\xb0a\xcc#\x17)\x1ci\x91R`t\x07I\x9b4\xf6\
\x89\xe3\x10\xfa\xef\xa5Y\xd6\xcb\x0fc\xb0\x16\xfa\x15\xeb\x83\xfc\xc1\x82\
\xd1\x80\x05,if\x99\xbc\xde\x12ceG=p{~K\x18F\x8f\xaf^\xbd\xfa\xfd\'\x9f\xfa\
\xe2\xc9\xcf\xfe\xf3\xcf\xf2\xc2\xf7_@~\xe3\xeb\xdf\xe0\xf3\x9f\xff\x15\xd6\
\xaf\xdd8\xe6{\xfe\'\xdb]\xb6^\x9c\xb6rj\xde\xd2\x8d\xb5\xb0F\x0b\xac\x11\
\xbdj\xa1\x01\xc3\xf8\x88\xc3\xcf>0\xc8\xcf\xed\x1d\xe2\xae-\x11\x83E\x07\
\xb7\x9fM\xd6X\x8c1\x18\x9d\x91e)I\x92\x90\xa6)\x99\xceV\xe2|\xe5i\xed\xcag\
\xac\x01\xdb\x07\xdf\xf7\x08\x18\xdaq\xc6\xa9K\rn,$\x91T\xde\xbdQ\x94\x7f\
\xf0\xcb_\xfeJ\xfe\xd1O=\x06\x80\xdc\xbcy\x0b\x7f\xf4\x87\x7f\xec\xe4r\xd1N\
\x8b\xb8\xfffM\x17&g\x04\xcb\xad\x9e\xdb\xad\xd5\xc2\xd8\x9e,\x90X\xa2@01\
\xecp\xfb\xc6\x80\xd1\xb2\x8bR|\xa8\nY\xcc\x87\x12Z\x1b\x8d\xd6}\xab[\xdb\
\xb3\xeb\x87\x12\xdfZ\x8d\xc5 \xac\x01L?\x8cz\xc0A#\xfa]\xbc\xb2\x14s\xf6jSt\
S1\x1e\x04\xc1\'\x86\x87W\xad\xfb\xd2\x97\xfe\x1d\xbf\xfc\xaf\xfe5r\xeb\xd6\
\xdbX\xb3f\xcd\x80\xe7\xba\xf7vb\xbb\xe9\xda\x9c\x91\xf3K\x964\xed\x11\xe8YS\
#\x84a\xfdj\x8f\x8f\xef*\xe0\xc8\x94\x93\x17\x16\xa8\xd4\xda}\xd9\xd0\xb7\
\xba1\xbd\x18\xd7f\x05\xb4\xe9\x83\xfe`X@\xaf\x84\x8c\xb0\x06D\x9f\x00\x06A\
\xaf\x8a\ta\xfb\x7f\x874\xd5\\\x9di3\xbf\x94\x85\xae\xe7\xef\xca\xe7\xf3w~\
\xees\x9f\xf7w\xef\xf9)\xe4\x9f\xfc\xc9\x1f\x10\x04\xfe\xb8\x90r\xf7R\xd3\
\x96nT\xb1\xad\xaeY\x01\xd4\xf7\x02\xc3%\xc9\xc3{\x8a<\xfe\xf12wm\xcdq\xf9F\
\x83\xb7\x8e\xddd\xbe\xd6F\xf7\x01\xa7iF\xb7\x9b\xd0lv\x88\xe3\xb4W\xa5,\x08\
!>4\xed\x8a\xf5\xb1\xbd\x84\xee\x91\xeeM0\x8c\x0e\x85l^["\xf4U?\xb1-\xb5z\
\xc2\xd4\xcd\x8e\xd0VMDa\xb8{\xf5\xd8\xf8\xc0\xbf\xf9\x95/\xe0<\xbc\xf7\x93\
\xaer\xd4\xc6T\xb3\xa9\xb2l\x9d\x85\xba\xb0\xbd.i\xc4\xad\xa6\x14x\x82;7\xe7\
\xd8\xb3-\xc7\xe8\xa0C\xce\x1f@g\x19\xc7\xcfW\xb1\xd6p\xff\x9d\xc3\x04\x9e`v\
~\x89+\xd7\x17\xb8<UcaY\xb3}\xf3\x08B\x80\x94\x12G)<O\xe2{.\xbe\'\x893K\xb7\
\x9b\xa1\xb5\xfe\x91\xf0\xf3]\xc9\xa65\xa3\xdc\xb7s\x94\x03Gop\xe4\xe4,q\xa2\
\xd1\x19\\\x9fks\xfb\x86\xa0\xe0y\xfe\xf60\x0cF\'\xd6\x8e\xcc9R9\xa1@l\xea&f\
Ue\x19\xd1\xeaXc\xb4\x16\x82\x95\xea\xc3\xd8\x90\xcf\x9em9\x86J\nk-Q \xd9\
\xb3}\x90LkN\x9c_\xa0\xd9\xec\xe0\x8a.\xc7\xce\\\xa7\xd96lZW\xe6\xcem#\x8c\r\
\xe7PJ\xe18\x0e\x9e\xe7\xe1\xf9\x92\xbb\xef\\\x8b\x1f\x168=Y\xe5\xec\xa5\x1a\
\xca\tp\x95X\xc9\r\xd7\x91L\x8c\xe6\xd8\xbd}\x88bN\xd2iu9~\xbeB7\xd6\xdc\xac\
uXld\xee\xf8\x80\xb36\n\xc3\x89G\x1ey\xf4\x8c\xe3:n\xc1Z\xd6ub\x9b_jb\x93\
\xb4\xdfi\xe9Y\xdfu\x05\x9b\xc6\x03\xd6\x8dz(i\xd1\xba\x97l\xa1/\xb8k[\x89\
\xa5z\x8b\x1f\xfc\xf0:\xf3\x95e\x96\x9b\x19C\xe5\x12{JC\xec\xbam\x98\xe1\x01\
\x1f\xc7Q\xb8\xae\x8b\xe7y\xf8\xbe\xcf\x9e;r\xe4B\x97Vc\x91\x8d\xe3k\xb9}\
\xeb8\xbe\xaf\xd0YF\x96i\x8c\xc9(\x17=\xf2>l_\x9f\xe7\xf1\x87\xd7a\xb1=C\xb5\
3\x96\x1a)\xe3\x83j\xc4\xf3\xfd\x89\xc1\xc1A\xdfq]\xa7ha\xb4\x9b\x18\xaf\xd1\
\x1163\x1a,\xc2`\xd0\xd60\x18\xbal\x18\xf7)\xe6\xc4J\xac\x9b~\xf9\x936\xc3\
\xa6-\x16\xaaM\x16\x1a\x1e\xda\x86\\\xafh^;|\x13\xd7\xb1\xec\xbd{\x8c\x91r\
\x8f\x84\xe7\xb9H\xa9\xb8~}\x9ew\x8f_b\xb0\x14\xf0\xc8\x03\xdbY?Q\xc6\x1aM\
\x9c$$qL\x1c\'\xa4Y\x8a\xd6\x19\xae\x03\xbb\xb7\x0f\xe3\xb9\x0e\x8eR\\\xb8Z\
\xa5\xde\xca0\xd6\xcd\xb9\xae;\x1a\xe5\xa2\xc0\xb1\xd6\x14\xac\xb1\x03\xdd\
\xc4\xc8n\xdc\xeb#}w\n\xac\xa5\x18\tV\r8(a\xfb\x16\xfa \xc1\xaf\xcfV\xf9\xe1\
\x89)f+\x1db\x9bC\xf7\x13\xf2\xe2T\xcc\x0b:\x03\xe0S\x0f\xacc,\x0c\xb1\x16&\
\xaf\xdcd\xff\xe1\xf3(\xc7a\xef}[Y\xbbz\x00c4I\x92\x10\xc71I\x1c\x93\xa4i\
\xaf\xeaI\x89R\x0eQ\xe4q\xef\xae\x1c\xad\x8e\xa6R\xab\xd3\xeadh#|\xc7q\x06}?\
\x08\x1cmLh\xac\rufE\x96\xd9\xbeF\xe9\x95E!,\x85H\x92\x0f\x05\xc6dhmV\x92.\
\xcd2&\xaf.p\xf6J\x9dz\xd7C\xa8[\xda\xc8\xa2\x8d\xe1\xf2t\x8b\xbf\xda7\x85r\
\x14\x8f~L\xd1nW9xd\x12!\x04\x0f\xee^\xcf\xe8P\x8e8\xee\xa2uF\x9a\xa6$IJ\x9a\
\xa6\xfdu\x05\xae\xeb\xe2\xfb>\xae\xeb\xd2\\l2_\xa9\xd2nw\xd0Z\x01()e\xceQ\
\x8e\xefXk\x15\xd6:\xb7\xd4e\x7f\x1b\xdb+s\x806\x19\xb5\xa5\x16\x0e\xa2\xef\
\x01\x8d\xc5\xd2\xe9\xa4\\\xb8\xba\xc8b\xd3b\xacB\xae4#z$\xb4\xe5\xeaL\x8b\
\xff\xf5\xc6\x15\xe6+\xcb\x98\xac\x8d\xeb\x08>\xb6{=\x81\x0b\x0b\xd5E\x80\
\x95\xde\x91\xf5\xcb\xa8\x10\x820\x0cp\xdd^\xceT\x97Z\xbc\xba\xef\x0c\xaf\
\x1e\xb8\xc4r\xc3"\xa4\x04\x81\x14B\xb8X\xab\x9c[\x15\xac\xa7LL\xdf\x03}i\
\x0cL\xdfl\xf2\xfa\xa1e\x02\xa7C\x96e\xa4\xfd0\xea\xc6\x19\x93SM:\x89D\x08z\
\xb1\'n5,\xdb\xaf\xf705\xdbb\xa1\xd6 \xe7[\xb6\xac+a\xcc\x14\xef\x9e\x9c"\
\xf0\x1c|O `E\x17!@\x08\xc5\xc4\xea!\xee\xd9\x95\xa3\xb3\xd4\xe2\xe57O\xf3\
\xfd\x1fL2\xb7$\x90\x8e\x87\x14\x12\x81\xb0Xk\xb5\xcep\x80.\x10K)\xac\x04\
\xd1k0}\x85(,\xb9\xd0e\xddx\x8eB\x98GkM\x9a\xf5B(N\x0c\xf5n\x8d\x1b\x8b\r\
\xb2\xb4\x07XX@X\x04 EO\xd8! N%\x99\x81\x93\x93u\xce\\\xaa\x11\xfa\x8a;7\xe7\
\x99\x18v{r\x01PJ\xe2y\x1eqf\xb8>\xdbdx\xb0\xca\x99\xc9Y^\xdaw\x89J]\xa2\x1c\
\x0fG\x81\xe3(\x10\x98\xcc\xe8n\x1c\xc7\xa9#\x04M\xb0u\xcf\x91\xd6s\xac\xa0\
\xbf\xc5\xb3\xd6 \x05\x0c\x16\x1cvo/3>\xe4\xf4\xdcl,J\n\xd2\x0c,\x82\xf3\xd7\
\x1atb\x8dU\xa2\xbf\x99\x01)A)\x81\x92\x02\xd9\x9fJ\x08,\n-<ry\x9f\xad\xebK\
\xac\x1d2}\x81\x08\x8e\xe3\x90/\x14Xn\tN]\\\xe2\xa5}\xe78wi\x91\xf9%\x89r|\
\x84\x05GY\xa2\xc0EI\xd24I\xea\xadV+v\xb46u\xb0\xf3\xbe\'\xd3(\x10\x01\x18{K\
\x93g\xd6Ro\xc5\xb4:\t\xc6\x08\xb0\xb7\x04\x96\xc0\xf7\x14\xebWG\x8c\x95]j\
\xf56X\x85\x10=r\xb7\xc0\xaf\x90\x10\x02%%R\x08\\W2>\x1c0\x98\x03a\xb3\x15\
\x0fH\xd1\xeb=\x8dF\x97\xd3\x17o\xb2\xb0,I\x8d\x87\xe3\x04 $Z[\x02\xcfR\xcc9\
Ha;\x9dn\\]Z^\xee\xc8n\x1c7\x8c17\x02\x8f\xf6@^\x08%\xf5\x8a\x88K\x93\x94\
\xd9J\x8b\x99\xf9\x16\xdd8!M\xb3\xde\xec\'\xf3\xfa\xd5\x05vo/S\xccI\xa40(\
\x01J\x82#A)z$\xa4\xc0Q=\x02JI\xcaE\x8f\xcd\xe3\x1e9\xdf\x80\x10=\x99\xe188\
\xae\x0bH*\xb56\xb5eCb#\x94\x13\xa1\x94\x8b\x10\n)%\xf9\xd0a \xefbM\xb6\xd4n\
5g\xe7\xe7\xe7c\xd9n\xb5ZY\x96]q\x94\xad\r\x15$\x81\x8b0\xba\xb7\x87\xed\xc6\
)s\x0b-._oPo&\xe8,%\xcb\xb2\x95d\x0e\x03\xc5\xfa\xb1\x80r\x1e<eW\xc2EJ\x81#\
\x05J\xd2\x03.\x15\x8e\x92x\xae\xc3P\xc9g\xb8$q\x1cVd\x86\xeby\x84A\x80\xd6\
\x82\xe9\xb9\x0e\x99\xf1q\x9d\x00!\x9c\x1ex\xd1\xfb\x8eR\xde\xa5TP:K\xe3\xd9\
\xa5\xa5\xe5\xe9S\xa7\xdeO\xe4|\xe5f\x9a$\xe9\x15\x89\xb962\xa0t\xb9 \x04h\
\x91\xa4\x19I\x92Ro&\x9c\xbfVg\xb6\xd2!I{5;KS\xe2n\xc2\xa5\xabs\x9c9?\xc5\
\xc4\xb0\xc3};W1>\x12\x12z\xb2g\xf1\xfe\xa2JI\x02\xcfaU9`\xdb\xda\x88\xc0\
\x15\xcc.dt\x12\x89\xeak\xa40\x08\x08\x82\x90\xcab\xcc\x8d\xb9\x0e\x88\x10)\
\xfb\x96\x17\x0e\x02\x07\xd7\x91\x8c\x0ey\xe4\x02:\xcdf\xf3\xf2Bua\x16\xd0\
\xce\x93O\xfc{\xf3\xd2+/^\x0f\xc3\xe0\xf4@.\xb8ob\xd8)^\x9e6\xc4qB\xa6{\x9d\
\xf5\xcat\x93\xd3\x97\x97)\x17\x8a\x04\x1e\xa4V0=\xd7\xe0\xd0\xd1)\x12-x\xfc\
\xe1m\x8c\x0c\x0fr\xe1Z\x9dsW\x97\x99\xabvY\\\x8eq\x1c\xc5P\xd1cx\xc0e\xdd*\
\x87\x91\x01AuY3u3E\n\xd8\xbe\xc1ch\xd0\'\x8aB\xba\t\x9c\xbf\xbc\xc8b\xc3A\
\xaa\x10a\x15\xb7v\xcd`\xc9\x87\xb0v\xd4G\x91Tk\xb5\xda\x99\xebSS\x0b\xfb\
\xdf<\x84\xba\xef\xbe\xfbY\xa8\xce\xa7Q\x18\x95\x02\xdf\xdbm\xac\x1c\xbe<\
\xd3\x11\xd5\xe5.\xda\xf4\x92\xb9\x1bg\xa4\xa9f\xb4\xec\x92\x0fajv\x91C\xef]\
\xa3\x13\x1b\x1e\xbc{\x03\xdb7\rQ.zL\x8c\x04l\x9c\xc81V\xf6\xe9\xc6)\x13#\
\x01\xf7\xee\xc8\xb1y\\0R\xd4DnF1\xa7P\xcaa\xa6b\xe8\xa6\x92\xd1\xe1\x1c\x85\
|\xc4\xb9K5\x0e\x1e]\xa0\x19\xe7Q*\xa4w\xe2#\xb1V \x85e\xc3\xb8\xc3\xae-~\
\x1a\xb7\x17\x8f\x9f=\x7f\xfe\xcf\x9f\x7f\xfe\x7f\\8}\xe6\x94q\xaeM]\xe1\xab\
_\xfdJ\xf7O\xff\xf4\xbb\xc7=?86>\xe4\xad\xdf<\xeeGS\xb3u\xd2\xb4W\xf33c\x98\
\xbc\xd1`\xff1K\xb5\xe6rej\x9e8\xb1\xdcw\xd7\x1a\xd6\x8f\x86\xa4I\x97\xa6N\
\x91RP\x0c$\xe3e\x18\x8c2J\x05\xc5\xaab\x86\xd1q\xcf\x18B\xe2\xb9\x96M\xe3\
\x1e\xae\xebsuVsj2\xa1\\Lx\xe7\xd8\x1c\x0bK\x1e\xd2\xcdcq\xc0\x8a\x9e\xfd\
\xad%\x1f\x19\xb6o\x08\x08\\\xbd8]\x99\x7f\xf7\xd2\xe4\xc5\x0b\xd6\xda\xcc\
\xf7=\xd4\xeb\x07^\xe5\xcf\xbf\xf7\x97\xccW\xe6\xdaa\x18\xe6\x83\xc0\xbd3\
\xf0\xdd\xa1\xe9JG\xd4\x96\x93\xde1\x07\x86$5,,u\xb8qs\x19)\xe0\xde\x9dcl\
\x9c\xc8!\xd0diF\x9ae}=\x93\xd0l\xc5\\\x9bm\xe0y\x92R\xce\xf6wu \xfbI\x1bE>c\
#9\\7\xe0\xf4d\x9b\xa3\xa7k\xdc\x983\x18QB*\x1f\xac\xc4"0V\xe0(\xc3\xf6\x8d\
\x0e{\xb6\x07I\xbbY=~\xfa\xf4\xa9\xef=\xf7\xdcwO>\xff\xfc\xff\xd433\xb3H\xd3\
\x81\xd7^\xfdk\x9ez\xea\x8b\xadF\xb3\xf1\x96\xce\x92C\xebF\xdc\xc6\xfdw\x0c2\
XT\xf6\xd6\xb1\x9f\xb5\x96zKsc\xc1\x92\xd8\x08\xdfs\xc9\xd2\x94n7\xa6\xd3\
\xe9\xd0\xe9t\xe8v\xbbt\xfb\x8a\xd2\x1a\xdb\xb7b\xaf\x129\x8e\x83\xeb\xba\
\x04\x81O\x14\x86\xe4\xa2\x80\xd0\xd74\x1a\x1dn.x\xa4v\x08!#\x8cQ\x18#\xfb\
\x13V\x95\x85\xdds[d<\x19\xcfOO\xdfx\xf3\xd4\xa9\xf7Ohm\xe2\xf7\xdf\x7f\xbf\
\xf7\xdd\x00G\xde{\x8f(\xca\xb1\xb8Xk\x14\x0b\x85,\x0c\xbcm\xe5b0\xd6\x8e\
\xad\x9c\x99o\x8bn\x92\xf57\xdb\x00\x82v\xaci\xb7\x13<G\x10\xfa\n\xd7\x95\
\xbdF%\x05J)\xb2\x0cf*1\xb9\\\xc8\xf8p\x80\xef\xbb\x84a@\x14E\xf8AD\x9cJ\xce\
N.\xb2\xff\x9d\x19.\xdf\xd0\x18QD\xc8\x00k{\xe0\xad\x95\x18c\x19(\x1a\x1e\
\xda\x13\xb2y\x8dj\xcd\xcfM\xef;r\xe4\xddg_x\xe1/\xcf\x97\xcbe\xdd\xe9t> \
\x00\xb0g\xcf\x1e\xb6l\xd9\xaa\xeb\xf5z\xcd\xf7=\xa7\x10y\xdb\x06K\xe1\xc0r+\
\x93\x95Z\x874\xd3\x08\x01B\n\xb2\xcc\xb2\xd4H\xa9.\xc7\xc4\xa9E\xc9\xde\x86\
\xc5\xf7=\x82 \xc0X\xc5\x8d\xf9\x0e\x85|\xc4\xba\xf1\x02a\xe0#\x94G\xbb\x0bS\
\xd3-\xde9>\xcf[Gj\\\x9bu\xd1b\x10!C\x8cQX\xdb\x0b\x1bc,\xb9Hs\xdfN\x8f\xbbo\
\x0b\x92Nk\xe1\xc4\x89\x13\xc7\xff\xfb\xa1C\x07\x0f\xcd\xce\xce6o\x81\xff\
\x11\x02\x87\x0e\x1d\xc4u%\xcdf\xa3S*\x96n\x06\xbe\x1f\r\x14\xfd\x8d\x83\xa5\
\xb0\xd0\xeahQ[\xee\x90f\xa6\xafs$\x08E+\x81\xcab\xca\\-\xa6\xba\x9cRof4;\
\x9a\xdaR\xcc\xd4L\x1b\x8d\x8f\x14\x82\x9b\x956\x17./s\xf4\xd4"\xef\x9e\xacs\
\xfe\x8a\xa1\xd1)!T\x19\xc4\x87\xc0\x1b\x81\xb1\x86BNs\xefN\x97\x07vE\x99\
\xb4\xcb\xe7O\x9f>\xf5\xdd\xfd\xfb\xdf|\xf1\xd8\xb1\xf7*Z\x9b\x0f\x9f\xd1|\
\xf4\x8a\xc9Z\xcb\x93O<)w\xef\xd9s\xfb\xd0\xf0\xf0\xaf\xbb^\xee\x97\xa6\xe6\
\xd3\xd5o\x1e\x9d\x13G\xcf-\xd0h\xa5\xfd\x13\x06\x85\xe7\xf4\x1a\x95\xa3\x04\
\xbe\x03\xa1\'\x88|\x89\x14\x10\'\x0e~P\xc0w4q7\xa1\xdd\x11tS\x0fcC\x90\x11\
\x08\x1f\xdb\x0f\x17k\xc1X\x10h\x86\xcb\x19\xf7\xefr\xb9\x7fg\x98:\xa2>y\xf6\
\xcc\xe9g_\x7f\xfd\xb5\xe7^~\xf9\xa5K\xe6\x96\xf2\xfb\xd0\xf8\xc8\x05\xc7\
\x81\xb7\xf6\xf3\xd4\x93\xbfi\xf7\xed\xffA-\x8ar3\xbe\xef\xba\xc3\x83\xd1\
\xea\x89\xd1|!\xf2]\xd9\xead\xc4io\xd7\xe4(\x85\xa3z\xc9)\xa5\x8b\xc5#\xd1\
\x1eI\xe6ceD\x9a\xb9\xb4\xba\x0eq\x1a\xa2) \x9c\x02RE\x80\x87\xb1j%Q\x8d\x01\
\xcfM\xed\xc65\x9aG\xef\x0f\xb9\xf7\xce \x96\xa6~\xee\xdc\xd9\xd3\xcf\xed\
\xdb\xf7\xe6\xf3/\xbf\xfc\xd7\x97*\x95\x9a~\xfa\xe9o\xfdm\xb8\x1f%p\xe5\xcaU\
\xfe\xe6o^\xe5\xcb_\xfe-\xf3\xdak\xaf.DQt5\xf4\xbdlh \x1aY\xbf\xbaP\\5\x94S\
\x00\xdd\xd8\x08c{b\xacw\xee\xe3\xf4\xa7\x87R^O\nH\x07)=\x84\xf0\x10\xd2\x05\
\xeb`l\xbf\xc2h\x811 \x85ah0\xe3\xfe\xbb\\>\xb3w\xc0\xee\xd8\xe85\xda\x8d\
\xca\xf1\x13\xef\x9fx\xf6\x8d7^\xff\x8b\x17_|\xf1\xd2\xcf\xff\xfc/d_\xf8\xc2\
\xaf~\x04\xfc\xff1\x84><~\xeb\xab_\xe5\xe2\xe4\xa4|\xe8\x81\x07\'6\xac\xdf\
\xf03\x03\x83\xe5_DywW\x97\xb3\xc1\xf3\xd7\xea\xce\xb9ku\xe6\xaa1\x9d\xd8b\
\xadD\x08\x07%\x9d\x9e~\x11\x0e\xe0\x80U\xfd\xd2\xa8\xd0Zbz\x07\xdd\xb8\x8ee\
\xb0(\xd8\xbc\xde\xe3\xde\x9d\x05\xbbcS.V\xb23w\xed\xea\xe5CG\x8e\xbc\xf7W\
\xfb\xf6\xed;p\xe0\xc0\xfe\x9b\x9b6m\xd4\x97/_\xf9\xb1\x18\xff\xde{\xe2_\xfb\
\xc2\xafr\xed\xda5\xb1w\xef\xc3\xa5\x8d\x1b7\xed.\x97\x87>\x13\x04\xe1^+\x9c\
\xadKM]\xba1\xdfU7\xe6:ba)\xa1\xd9\xb1\xc4\tdY\xcf\xca\xa0z1\x8eB\xa0\xf0\
\x1cI.P\x0c\rz\xac\x9f\x08\xd9\xbe)o7\xae\xcd%\xb9\xc0,T\xab\x953\xe7\xce\
\x9e\xdd\xff\xf6;o\xbf\xf1\xf6\xdbo\x9f=s\xe6\xecr\xa1P0\x8b\x8b\xb5\xbf\x13\
\xdf?\xea\xa7\x06\x8f}\xf2Q\xe7S\x8f=6:\xb6z\xf5O\x95J\x03\x9f\x08\xc3\xf0\
\x1e\xa9\xdcMI&\xca\xcd\x8e\t\xeb-\xad\x9am-\xda]M\x9a\xf6\xf6\xc4 \x90\xca!\
\xf0]Jy\x97\xe1\xc1\xc0\x8c\x8eD\xd9\xd0`\xd0r\x95\xad4\xea\xcb\x17\xae^\xbd\
z\xe4\xf8\x89\x13o\xef\xdb\xbf\xef\xf4\xa1\x83\x87*i\x9a~\xe4.\xec\x9f\x84\
\xc0\xad\xf1\xe9\x9f~\xcc{\xe0\x81\x87FFV\x8dn+\x95\x8awFa\xb8\xc3\xf3\xfd\
\x8dR\xb9cR\xaa\x01\x84\x8a@\xb8B\x08)\xa4\xb0\x12a\x84$\x91\xc2\xb6\x94\xa4\
f13\xedv\xeb\xca\xfc\\\xe5\xdc\x85\xc9\xc9\xd3\xef\x1e>|\xf9\xe0\xc1\x83\x0b\
\xcdV\xeb\x1f\x0c\xfc\'"pk<\xbc\xf7\x13j\xdb\xb6\xed\xb9U\xabV\x95\xf3\xf9\
\xdcX\x14Eca\x18\xad\n|\xbf,\x95\xca\x0b!\\\xc0\x1ac\xd28\xee6\xda\xedv\xad\
\xdeh\xcc\xd7\x16\x17o\xceLO\xcf\x9f\xbfp\xa1v\xec\xe8\xd1\x16\xf0\x91\xf2\
\xf8\xff\x85\xc0\x87G\xb1X\x94\xb7\xddv\x9b3<4\xe4\xe7ry_9\xca3\xd6*\xa3\rY\
\x96\xe9F\xa3\x9e.,T\xe2\xe9\xe9\x99\xb8Z\xad\xa6\xf4.\x04~\xe2\xf1\xbf\x01\
\xbb-\xf6\x84\xfe\xdc\x84\xdf\x00\x00\x00\x00IEND\xaeB`\x82\xd8\xf8]\x9d'