# -*- coding: UTF-8 -*-
#License: latest GPL

import gettext, locale, os, optparse, sys
from data.info import INFO

def fix_python_path(phatch_python_path=None):
    if not phatch_python_path:
        phatch_python_path = os.path.dirname(os.path.abspath(__file__))
    if phatch_python_path not in sys.path:
        sys.path.insert(0,phatch_python_path)
    return phatch_python_path

def load_locale(app,path,canonical='default',unicode=True):
    locale.setlocale(locale.LC_ALL, '')
    if canonical == 'default':
        default_locale  = locale.getdefaultlocale()[0]
        if default_locale is None:
            #for mac
            canonical   = 'en'
        else:
            canonical   = default_locale[:2]
    gettext.install(app, path, unicode=unicode)
    i18n = gettext.translation(app, path, languages=[canonical],fallback=1)
    i18n.install()

def parse_options():
    parser  = optparse.OptionParser(
        usage   = "%(name)s [options] [actionlist] [image files/folders]"%INFO, 
        version = "%(name)s %(version)s"%INFO,
    )
    parser.add_option("-c", "--console", action="store_false", 
        dest    = "console", 
        help    = _("Run %s as console program without a gui.")%INFO['name'])
    return parser.parse_args()
    
def main(paths):
    phatch_path     = fix_python_path(paths.get("PHATCH_PYTHON_PATH",None))
    fix_python_path(os.path.join(phatch_path,'core','lib'))
    load_locale(INFO['name'].lower(),paths["PHATCH_LOCALE_PATH"])
    options, paths  = parse_options()
    
    if options.console:
        #console version
        from console import console as run
    else:
        #gui version
        fix_python_path(os.path.join(phatch_path,'pyWx','lib'))
        from pyWx import gui as run
        
    run.main(paths)
    
if __name__=='__main__':
    main()
