"""All PIL related issues."""

#license latest GPL
import os

import Image

IMAGE_DEFAULT_DPI       = 72

#from core import ct
#from core.translation import _t
_t =str
def extract_dpi(info,image,prefix):
    #info
    image_info              = image.info
    if image_info.has_key('dpi'):
        dpi                 = image_info['dpi']
        if len(dpi) > 1:
            dpi             = dpi[0]
        info[prefix+_t('dpi')]  = dpi
    else: dpi               = 0
    if dpi == 0:
        if image_info.has_key('aspect'):
            info[prefix+_t('dpi')]  = dpi = image_info['aspect'][0]
        else:
            dpi             = 0
        if dpi == 0:
            info[prefix+_t('dpi')]  = IMAGE_DEFAULT_DPI
        
def extract_info(info, layer, location,prefix=''):
    image                   = layer.image
    dpi                     = extract_dpi(info,image,prefix)
    #retrieve properties
    folder, filename        = os.path.split(location)
    filename, typ           = os.path.splitext(filename)
    typ                     = typ[1:]
    width, height           = size = image.size
    mode                    = image.mode
    #summarize all photo properties in info
    info.update({
        prefix+_t('location')   : location,
        prefix+_t('folder')     : folder,
        prefix+_t('filename')   : filename,
        prefix+_t('type')       : typ,
        prefix+_t('width')      : width,
        prefix+_t('height')     : height,
        prefix+_t('size')       : size,
        prefix+_t('mode')       : mode,
        prefix+_t('format')     : image.format,
    })
    return info

def create_test_info():
    global TEST_INFO
    im              = Image.new('L',(1,1))
    TEST_INFO    = extract_info({},Layer(im),'/test/unknown.png')
    return TEST_INFO
       
def report_invalid_image(f,valid,invalid):
    try:
        im  = Image.open(f)
        im.verify()
        valid.append(f)
    except:
        invalid.append(f)
            
class Photo:
    def __init__(self,filename):
        #validate image
            name = self.current_layer_name  = _t('background')
            layer                           = Layer(Image.open(filename))
            self.layers                     = {name:layer}
            self.init_info(layer,filename)
        
    def get_filename(self,folder,filename,typ):
        self.info[_t('new')+'_'+_t('width')], \
            self.info[_t('new')+'_'+_t('height')] = \
            self.image.size
        return os.path.join(folder,
            '%s.%s'%(filename,typ)).replace('<','%(').replace('>', ')s')%\
            self.__dict__
            
    def init_info(self,layer,location,prefix=_t('new')+'_'):
        self.info_original  = {}
        extract_info(self.info_original,layer,location)
        self.info           = self.info_original.copy()
        extract_info(self.info,layer,location,prefix=prefix)
        
    def get_info(self):
        """Use method for read-only"""
        return self.info.copy()
            
    def get_info_original(self):
        """Use method for read-only"""
        return self.info_original.copy()
            
    #---layers
    def get_layer(self,name=None):
        if name is None:
            name   = self.current_layer_name
        return self.layers[name]
    
    def set_layer(self,layer,name=None):
        if name is None:
            name   = self.current_layer_name
        self.layers[name]   = layer
        
    #---image operations affecting all layers
    def save(self,filename,*args,**keyw):
        """Saves a flattened image"""
        #todo: flatten layers
        layer   = self.get_layer()
        layer.image.save(filename,*args,**keyw)
        if hasattr(keyw,'dpi'):
            self.info[_t('new')+'_'+_t('dpi')] = keyw['dpi'][0]
        extract_info(self.info,layer,filename)
        
    def convert(self,mode,*args,**keyw):
        """Converts all layers to a different mode."""
        for name, layer in self.layers.items():
            layer.image = layer.image.convert(mode,*args,**keyw)
            self.set_layer(layer,name)
        self.info[_t('new')+'_'+_t('mode')] = mode
            
    def resize(self,size,method):
        """Resizes all layers to a different size"""
        size    = (max(1,size[0]),max(1,size[1]))
        for name, layer in self.layers.items():
            layer.image = layer.image.resize(size,method)
            self.set_layer(layer,name)
        self.set_size(size)
            
    def set_size(self,size):
        self.info[_t('new')+'_'+_t('width')], \
            self.info[_t('new')+'_'+_t('height')]   = size
        self.info[_t('new')+'_'+_t('size')]         = size

class Layer:
    def __init__(self,image,position=(0,0)):
        self.image      = image
        self.position   = position
        
if __name__ == '__main__':
    print create_test_info()