# -*- coding: UTF-8 -*-

#---first test
import os

try:
    #phatch environment
    import core
except ImportError:
    #test environment
    import gettext, sys
    gettext.install("test")
    sys.path.insert(0,os.path.dirname(os.getcwd()))
    
#---begin
import wx
#license: GPL
from core import base, ct

#core.lib
import formField 
from core.lib.events import Receiver, send

#gui-dependent
import graphics, popup, vlist #popup only for popup.wildcardlist
from pyWx.wxGlade   import dialogs

VLIST_ICON_SIZE = (48,48)

class _IconDialog:
    if wx.Platform == '__WXGTK__':
        _icon_size  = (48,48)
    else:
        _icon_size  = (32,32)
    def _icon(self,name='information'):
        name    = 'ART_%s'%name.upper()
        #title icon
        bitmap  = graphics.bitmap(name,(16,16))
        _ic = wx.EmptyIcon()
        _ic.CopyFromBitmap(bitmap)
        self.SetIcon(_ic)
        #dialog icon
        bitmap  = graphics.bitmap(name,self._icon_size)
        self.icon.SetBitmap(bitmap)
        self.icon.Show(True)

class ErrorDialog(dialogs.ErrorDialog,_IconDialog):
    def __init__(self,parent,message,details,ignore=True,**keyw):
        super(ErrorDialog,self).__init__(parent,-1,**keyw)
        self._icon('error')
        self.message.SetLabel(message)
        self._details(details)
        if not ignore: 
            self.ignore.Hide()
            self.skip.SetDefault()
        
    #---events
    def on_skip(self,event):
        self.EndModal(wx.ID_FORWARD)
    
    def on_abort(self,event):
        self.EndModal(wx.ID_ABORT)
    
    def on_ignore(self,event):
        self.EndModal(wx.ID_IGNORE)
        
    #---details
    def _details(self,details):
        #self.details_show(False)
        self.details.SetValue(details.strip())
        
    def details_show(self,show):
        sizer   = self.GetSizer()
        self.details.Show(show)
        self.SetMinSize(sizer.GetMinSize())
        self.Fit()
        
    def on_details(self,event):
        self.details_show(event.IsChecked())
    
class ExecuteDialog(dialogs.ExecuteDialog):
    def __init__(self,parent,drop=False,**options):
        super(ExecuteDialog,self).__init__(parent,-1,**options)
        self.set_drop(drop)
        if wx.Platform == '__WXGTK__':
            width = 600
        else:
            width = 450
        self.SetSize((width,self.GetSize()[1]))
        
    def browse_files(self):
        dlg = wx.FileDialog(
            self, message   = _("Choose File(s)"),
            defaultDir      = self.get_default_path(),
            defaultFile     = "",
            wildcard        = self.wildcard(),
            style           = wx.OPEN | wx.MULTIPLE | wx.CHANGE_DIR |wx.FD_PREVIEW
            )
        if dlg.ShowModal() == wx.ID_OK:
            self._path      = ct.PATH_DELIMITER.join(dlg.GetPaths())
            self.path.SetValue(self._path)
        dlg.Destroy()
            
    def browse_folder(self): 
        dlg = wx.DirDialog(self,_("Choose an image folder"),
            defaultPath = self.get_default_path(),
            style       = wx.DEFAULT_DIALOG_STYLE)
        if dlg.ShowModal() == wx.ID_OK:
            self._path  = dlg.GetPath()
            self.path.SetValue(self._path)
        dlg.Destroy()
        
    def get_default_path(self):
        path    = self.path.GetValue().split(ct.PATH_DELIMITER)[0] 
        return os.path.dirname(path)
    
    def export_settings(self,settings):
        settings['path']       = self.get_path()
        settings['extensions'] = [self.extensions.GetString(i) \
            for i in range(self.extensions.GetCount()) \
            if self.extensions.IsChecked(i)]
        settings['recursive']  = self.recursive.GetValue()
        settings['stop_for_errors'] = self.stop_for_errors.GetValue()
        settings['overwrite_existing_images'] = \
            self.overwrite_existing_images.GetValue() 
        settings['check_images_first'] = self.check_images_first.GetValue() 
                
    def get_path(self):
        path    = self.path.GetValue()
        n       = len(path)
        if n and self._path[:n] == path:
            return self._path
        else:
            return path
        
    def get_selected_extensions(self):
        result  = []
        exts    = formField.IMAGE_READ_EXTENSIONS
        for index, extension in enumerate(exts):
            if self.extensions.IsChecked(index):
                result.append(extension)
        return result
                
    def set_drop(self,drop):
        if drop:
            #change title
            self.SetTitle(_('Drag & Drop'))
            #radio box
            self.source.Hide()
            #hide browse & path
            self.browse.Hide()
            self.path.Hide()
            #layout (only allow vertical fit)
            grid_sizer  = self.GetSizer()
            size        = (self.GetSize()[0], grid_sizer.GetMinSize()[1])
            self.SetMinSize(size)
            self.Fit()
        
    def import_settings(self,settings):
        #path
        self._path  = settings['path']
        self.path.SetValue(self._path)
        #extensions
        exts    = formField.IMAGE_READ_EXTENSIONS
        self.extensions.Set(exts)
        for index, extension in enumerate(exts):
            if extension in settings['extensions']:
                self.extensions.Check(index)
        #overwrite existing images
        self.overwrite_existing_images.SetValue(
            settings['overwrite_existing_images'])
        #overwrite existing files
        self.check_images_first.SetValue(
            settings['check_images_first'])
        #recursive
        self.recursive.SetValue(settings['recursive'])
        #errors
        self.stop_for_errors.SetValue(settings['stop_for_errors'])

    #---wildcard
    def wildcard(self):
        extensions  = self.get_selected_extensions()
        selected    = popup.wildcard_list(_('All selected image types'),
                        extensions)
        default     = popup.wildcard_list(_('All default image types'),
                        formField.IMAGE_EXTENSIONS)
        result      = [selected,default]
        result.extend([('%s '+_('images')+'|%s|')\
                    %(ext,base._wildcard_extension(ext))
                    for ext in extensions])
        return ''.join(result)
        
    #---events
    def on_browse(self,event):
        if self.source.GetSelection() == 0:
            self.browse_folder()
        else:
            self.browse_files()
        
    def on_default(self,event):
        exts    = formField.IMAGE_READ_EXTENSIONS
        for index, extension in enumerate(exts):
            if extension in formField.IMAGE_EXTENSIONS:
                self.extensions.Check(index,True)
            else:
                self.extensions.Check(index,False)
                
    def on_source(self,event):
        self.browse.SetLabel(_('Browse')+' '+event.GetString())
        
class FilesDialog(dialogs.FilesDialog,_IconDialog):
    def __init__(self,parent,message,title,files,icon='warning',**keyw):
        super(FilesDialog,self).__init__(parent,-1,
            style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER|wx.MAXIMIZE_BOX,
            **keyw)
        self.SetTitle(title)
        self.message.SetLabel(message)
        self.list.InsertColumn(0, _("File"))
        self.list.InsertColumn(1, _("Folder"))
        for index, f in enumerate(files):
            index = self.list.InsertStringItem(index, os.path.basename(f))
            self.list.SetStringItem(index, 1, os.path.dirname(f))
        self.list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
        self.list.SetColumnWidth(1, wx.LIST_AUTOSIZE)
        min = 100
        if self.list.GetColumnWidth(0) < min:
            self.list.SetColumnWidth(0, min)
        self._icon(icon)
        
class ProgressDialog(wx.ProgressDialog,Receiver):
    """+1 is added because eg opening a file is also an action"""
    def __init__(self,parent,title,message,maximum=1):
        wx.ProgressDialog.__init__(self,
                title   = title,
                message = message,
                maximum = maximum, #check +1
                parent  = parent,
                style   = wx.PD_CAN_ABORT
                            | wx.PD_APP_MODAL
                            | wx.PD_REMAINING_TIME
                            | wx.PD_SMOOTH
            )
        Receiver.__init__(self,'progress')
        self.subscribe('close')
        self.subscribe('set_index')
        self.subscribe('update')
        self.subscribe('update_filename')
        self.subscribe('update_index')
        self.Bind(wx.EVT_CLOSE,self.close,self)
        self.amount = 1
        
    #---pubsub event methods
    def close(self):
        self.unsubscribe_all()
        self.Destroy()
        
    def update(self,result,value,**message):
        """Fix for wxPython2.6"""
        status = self.Update(value,**message)
        if len(status) == 2:
            result['keepgoing'], result['skip'] = status
        else:
            result['keepgoing'], result['skip'] = status
        if result['keepgoing']:
            self.Refresh()
        else:
            self.close()

    def update_filename(self,result,index,filename):
        dirname, basename   = os.path.split(filename)
        message             = "%s: %s\n%s: %s"%(_('In'),dirname,_('File'),
                                basename)
        self.update(result,index*self.amount,newmsg=message)
        
    def update_index(self,result,index,sub_index):
        self.update(result,index*self.amount + sub_index + 1)
        
    def set_index(self,amount):
        self.amount = amount+1
        
        
class VListDialog(vlist.Dialog):
    def __init__(self,actions,*args,**keyw):
        super(VListDialog,self).__init__(*args,**keyw)
        self.SetIconSize(VLIST_ICON_SIZE)
        self.SetList(actions)
        self.Bind(wx.EVT_CONTEXT_MENU, self.OnContextMenu)
        
    def OnContextMenu(self,event):
        self.id_view_source = wx.NewId()
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.id_view_source,_("View Source"))
        item.SetBitmap(graphics.bitmap('ART_FIND',(16,16)))
        self.Bind(wx.EVT_MENU, self.OnViewSource, id=self.id_view_source)
        menu.AppendItem(item)
        self.PopupMenu(menu)
        menu.Destroy()
        
    def OnViewSource(self,event):
        import actions
        action      = self.actions[self.list.GetSelection()]
        module      = action.__module__.split('.')[-1]
        filename    = os.path.join(ct.ACTIONS_PATH,'%s.py'%module)
        message     = open(filename).read()
        dir, base   = os.path.split(filename)
        send.frame_show_scrolled_message(message,'%s - %s'%(base,dir),
            size=(600,300))
        
    def SetList(self,actions):
        self.actions    = actions.values()
        self.actions.sort(cmp=lambda x,y: cmp(_(x.label),_(y.label)))
        self.Clear()
        self.SetItemCount(len(actions))
        
    def GetItem(self, n):
        action  = self.actions[n]
        return (_(action.label),_(action.__doc__), 
            graphics.bitmap(action.icon,self.GetIconSize()))
    
    def GetStringSelection(self):
        return self.actions[self.list.GetSelection()].label
        

class WritePluginDialog(dialogs.WritePluginDialog,_IconDialog):
    def __init__(self,parent,message,**keyw):
        super(WritePluginDialog,self).__init__(parent,-1,**keyw)
        path    = os.path.join(ct.PATH,'templates', 'action.py')
        self._icon('information')
        self.message.SetLabel(message)
        self.path.SetLabel('%s: %s'%(_('Path'),path))
        self._code(path)
        self.template_show(False)
        
    def _code(self,path):
        self.code.SetValue(open(path).read())
        self.code.SetMinSize((660,300))
        self.code.SetFont(wx.Font(10, wx.TELETYPE, wx.NORMAL, wx.NORMAL, 0, ""))

    #---events
    def on_help(self,event):
        import webbrowser
        url = ct.SEND_MAIL%('Writing an action plugin for Phatch',
                "As I know PIL, I'd like to write a plugin which does ... "
                "but I still have this question: ... "
            )
        webbrowser.open(url)
        
    def on_template(self,event):
        self.template_show(event.IsChecked())

    def template_show(self,show):
        sizer   = self.GetSizer()
        self.path.Show(show)
        self.code.Show(show)
        self.SetMinSize(sizer.GetMinSize())
        self.Fit()
        self.code.ShowPosition(0)
 
def test():
    class App(wx.App):
        def OnInit(self):
            wx.InitAllImageHandlers()
            frame = wx.Frame(None, -1, "")
            self.SetTopWindow(frame)
            frame.Show()
            wx.CallAfter(self.show_dialogs)
            return 1
        
        def show_dialogs(self):
            self.show_error_dialog()
            self.show_execute_dialog()
            self.show_files_dialog()
            self.show_progress_dialog()
            self.GetTopWindow().Destroy()
            
        def show_error_dialog(self):
            dlg = ErrorDialog(self.GetTopWindow(),'message','details')
            dlg.ShowModal()
            dlg.Destroy()
            
        def show_execute_dialog(self,drop=False):
            dlg = ExecuteDialog(self.GetTopWindow(),drop)
            dlg.ShowModal()
            dlg.Destroy()
            
        def show_files_dialog(self,drop=False):
            dlg = FilesDialog(self.GetTopWindow(),'message','title',
                ['path/file'],'warning')
            dlg.ShowModal()
            dlg.Destroy()
            
        def show_progress_dialog(self,drop=False):
            from core.lib.events import send
            import time
            n   = 5
            dlg = ProgressDialog(self.GetTopWindow(),'title','messages',n)
            for value in range(n):
                send.progress_update(value)
                if not dlg.keepgoing:
                    break
                time.sleep(1)
            dlg.Destroy()
            
    app = App(0)
    app.MainLoop()
    
if __name__ == '__main__':
    test()