# -*- coding: UTF-8 -*-
"""Todo 
* create more actions
* backgroundcolor SetItemBackgroundColor(item, col)
* console option (make execution gui independent)
* automatic updating of properties
* save file lists
"""
#---Global import

#import & check wx
from wxcheck import ensure
wx = ensure('2.8','2.6')

#standard library
#import cPickle, gettext, os, time
    
#---Local import

#gui-independent
from core import ct#base, ct, common
from core.lib.events import Receiver #core.lib

#gui-dependent
from pyWx import images
import droplet, graphics
from pyWx.wxGlade import frame

#---Functions
def findWindowById(id):
    return wx.GetApp().GetTopWindow().FindWindowById(id)


class Frame(frame.Frame,Receiver):
    def __init__(self,*args,**keyw):
        frame.Frame.__init__(self,*args,**keyw)
        self._theme()
        self._loadSettings()
        self._menu()
        self._toolBar() 
        self.on_menu_file_new()
        self._icon()
        self._title()
        self._description()
        self._drop()
        self._events()
        self._pubsub()
        
    def _theme(self):
        if wx.Platform != '__WXGTK__':
            self.set_theme('nuovext')
        
    def set_theme(self,name='default'):
        if name == 'nuovext':
            from pyWx.nuovext import Provider
            wx.ArtProvider.Push(Provider())
        
    def _description(self):
        self.show_description(False)
        
    def _drop(self):
        dt = droplet.FileDropTarget(self,self.on_drop)
        self.tree.SetDropTarget(dt)
        
    def _icon(self):
        _ic = wx.EmptyIcon()
        _ic.CopyFromBitmap(graphics.bitmap('ART_PASTE',(16,16)))
        self.SetIcon(_ic)
        
    def _menu(self):
##        #---bitmaps
##        self.menu_view_description.SetBitmap(
##            graphics.bitmap('ART_TIP',(16,16)))
        #file history
        self.filehistory = wx.FileHistory()
        self.filehistory.UseMenu(self.menu_file)
        self._set_file_history(self.settings['file_history'])
        self.Bind(wx.EVT_MENU_RANGE, self.on_menu_file_history, 
            id=wx.ID_FILE1, id2=wx.ID_FILE9)
        #shell
        self.shell = None
        #menu_item (for enabling/disabling)
        edit    = [getattr(self,attr) for attr in dir(self) 
            if attr[:10] == 'menu_edit_']
        edit.remove(self.menu_edit_add)
        self.menu_item  = [
            (self.menu_edit,[item.GetId() for item in edit]),
            (self.menu_view,(self.menu_view_droplet.GetId(),
                self.menu_view_description.GetId())),
            (self.menu_tools,(self.menu_tools_execute.GetId(),)),
        ]
        self.on_show_droplet(False)
        # Mac  tweaks
        if wx.Platform == "__WXMAC__":
            #todo: about doesn't seem to work: why?!
            app = wx.GetApp()
            #app.SetMacHelpMenuTitleName(_('&Help'))
            app.SetMacAboutMenuItemId(wx.ID_ABOUT)
            app.SetMacExitMenuItemId(wx.ID_EXIT)
            #app.SetMacPreferencesMenuItemId(wx.ID_PREFERENCES)

        
    def _title(self):
        path, filename = os.path.split(self.filename)
        self.SetTitle(ct.FRAME_TITLE%(self.dirty,filename))
        self.frame_statusbar.SetStatusText(path)
        
    #---toolBar
    def add_tool(self,bitmap,label,tooltip,method,item=wx.ITEM_NORMAL):
        id      = wx.NewId()
        bitmap  = graphics.bitmap(bitmap,self.tool_bitmap_size,
                    client=wx.ART_TOOLBAR)
        args    = (id, label, bitmap, wx.NullBitmap,item, 
                    tooltip, "")
        tool    = self.frame_toolbar.AddLabelTool(*args)
        self.Bind(wx.EVT_TOOL, method, id=id)
        return tool
        
    def _toolBar(self):
        self.tool_bitmap_size   = (32,32)
        self.frame_toolbar = wx.ToolBar(self, -1)
        self.SetToolBar(self.frame_toolbar)
        self.frame_toolbar.SetToolBitmapSize(self.tool_bitmap_size)
        self.tools_item = []
        self.add_tool('ART_FILE_OPEN',_("Open"),
            _("Open an action list"),self.on_menu_file_open)
        tools_item  = [
            self.add_tool('ART_EXECUTABLE_FILE',_("Execute"),
                _("Execute the action"),self.on_menu_tools_execute)
        ]
        self.frame_toolbar.AddSeparator()
        self.add_tool('ART_ADD_BOOKMARK',_("Add"),
            _("Add an action"),self.on_menu_edit_add),
        tools_item.extend([
            self.add_tool('ART_DEL_BOOKMARK',_("Remove"),
                _("Remove the selected action"),self.on_menu_edit_remove),
            self.add_tool('ART_GO_UP',_("Up"),
                _("Move the selected action up"),self.on_menu_edit_up),
            self.add_tool('ART_GO_DOWN',_("Down"),
                _("Move the selected action down"),self.on_menu_edit_down),
        ])
        self.frame_toolbar.AddSeparator()
        self.toolbar_description = self.add_tool('ART_TIP',_("Description"),
            _("Show description of the action list"),
            self.on_menu_view_description,item=wx.ITEM_CHECK)
        tools_item.extend([self.toolbar_description])
        self.tools_item = [tool.GetId() for tool in tools_item]
        self._menu_toolbar_state = True
        self.frame_toolbar.Realize()
            
    def enable_actions(self, state=True):
        if state != self._menu_toolbar_state:
            for tool in self.tools_item:
                self.frame_toolbar.EnableTool(tool, state)
            for menu, items in self.menu_item:
                for item in items:
                    menu.Enable(item,state) 
            self._menu_toolbar_state = state
            self.tree.Show(state)
            self.logo.Show(not state)
            self.show_description(state and self.settings['description'])

#---menu events
    def on_menu_file_new(self, event=None):
        self._set_filename(ct.UNKNOWN)
        self.description.SetValue(ct.ACTION_LIST_DESCRIPTION)
        self._new()
        self.enable_actions(False)
        
    def on_menu_file_open(self, event):
        dlg = wx.FileDialog(self,
            message     = _('Choose an Action List File...'),
            defaultDir  = os.path.dirname(self.filename),
            wildcard    = ct.WILDCARD,
            style       = wx.OPEN,
        )
        if dlg.ShowModal() == wx.ID_OK:
            filename    = dlg.GetPath()
            self._open(filename)
        dlg.Destroy()
        
    def on_menu_file_save(self, event):
        if self.filename == ct.UNKNOWN:
            self.on_menu_file_save_as()
        else:
            self._save()
        
    def on_menu_file_save_as(self, event=None):
        dlg = wx.FileDialog(self, 
            message     = _('Save Action List As...'),
            defaultDir  = os.path.dirname(self.filename),
            wildcard    = ct.WILDCARD,
            style       = wx.SAVE,
        )
        if dlg.ShowModal() == wx.ID_OK:
            path        = dlg.GetPath()
            if dlg.GetFilterIndex() == 0 and not os.path.splitext(path)[1]:
                path    += '.py'
            self._save(path)
        dlg.Destroy()
        
    def on_menu_file_quit(self, event):
        self.on_close()
        
    def on_menu_file_history(self,event):
        # get the file based on the menu ID
        filenum     = event.GetId() - wx.ID_FILE1
        filename    = self.filehistory.GetHistoryFile(filenum)
        self._open(filename)
        
    def on_menu_edit_add(self, event): 
        dlg = dialogs.VListDialog(common.ACTIONS,self,-1,size=(400, 400),
            title="%(name)s "%ct.INFO+_("actions"), )
        if dlg.ShowModal() == wx.ID_OK:
            self.set_dirty(True)
            label   = dlg.GetStringSelection()
            self.tree.append_form_by_label_to_selected(label)
            self.enable_actions(True)
        dlg.Destroy()
        
    def on_menu_edit_remove(self, event): 
        if self.tree.remove_selected_form():
            if self.tree.has_forms():
                self.set_dirty(True)
            else:
                self.enable_actions(False)
                self.set_dirty(False)
        
    def on_menu_edit_up(self, event):
        self.set_dirty(True)
        self.tree.move_selected_form_up()
        
    def on_menu_edit_down(self, event):
        self.set_dirty(True)
        self.tree.move_selected_form_down()
        
    def on_menu_edit_enable(self,event):
        self.tree.enable_selected_form(True)
        
    def on_menu_edit_disable(self,event):
        self.tree.enable_selected_form(False)
        
    def on_menu_view_droplet(self,event):
        self.show_droplet(event.IsChecked())
        
    def droplet_label_format(self,x):
        return x[:11].replace('_',' ')

    def on_menu_view_description(self, event):
        self.show_description(event.IsChecked())
            
    def on_menu_tools_execute(self, event):
        base.apply_actions(self.tree.export_forms(),self.settings)
        
    def on_menu_view_expand_all(self, event): 
        self.tree.expand_forms()

    def on_menu_view_collapse_all(self, event): 
        self.tree.collapse_forms()

    def on_menu_tools_show_log(self,event):
        if os.path.exists(ct.ERROR_LOG_PATH):
            log_file    = open(ct.ERROR_LOG_PATH)
            msg         = log_file.read()
            log_file.close()
        else:
            msg         = _ ('Nothing has been logged yet.')
        self.show_scrolled_message(msg,'%s - %s'%(_('Log'),ct.ERROR_LOG_PATH))
        
    def on_menu_tools_python_shell(self,event):
        import shell
        self.tree.close_popup()
        if self.shell is None:
            self.shell = shell.Frame(self,
                title   = '%(name)s '%ct.INFO+_('Shell'),
                intro   = '%(name)s '%ct.INFO,
                values  = {
                    '%s_%s'%(ct.TITLE,_('application')) : wx.GetApp(),
                    '%s_%s'%(ct.TITLE,_('frame'))       : self,
                    '%s_%s'%(ct.TITLE,_('actions'))     : self.tree.export_forms,
                    }
            )
        self.shell.Show(event.IsChecked())

    def on_menu_help_translate(self, event): 
        self.show_message('\n'.join([
            _('If you want to translate Phatch in your language,')+\
            "\n%s %s."%(_('please email'),ct.CONTACT)
        ]), style = wx.OK|wx.ICON_INFORMATION)
        
    def on_menu_help_plugin(self, event): 
        dlg = dialogs.WritePluginDialog(self,'\n'.join([
            _('You only need to know PIL to write a plugin for Phatch,')+' ',
            _('as Phatch will generate the user interface automatically.'),
            _('Study the action plugins in:')+' '+ct.ACTIONS_PATH,
            '',
            _('If you want to contribute a plugin for Phatch,')+' '+\
            "%s %s."%(_('please email'),ct.CONTACT)
        ]))
        dlg.ShowModal()
        
    def on_menu_help_about(self, event): 
        import about
        from data.info import all_credits
        dlg = about.Dialog(self,
            title                   = '%(version)s'%ct.INFO,
            logo                    = graphics.bitmap(images.LOGO),
            description             = ct.INFO['description'],
            website                 = ct.INFO['url'],
            credits                 = all_credits(),
            license                 = ct.LICENSE,
        )
        dlg.ShowModal()
        dlg.Destroy()
        
    #---helper
    def _new(self):
        self.set_dirty(False)
        self.tree.delete_all_forms()
        
    def _open(self, filename):
        self._new()
        if os.path.exists(filename):
            try:
                data    = common.open_data(filename)
            except:
                self.show_error(
                    _('Sorry, the action list seems to be incompatible\n'
                    'with the current version (%(version)s) of %(name)s.')%\
                    ct.INFO)
                return
            self.description.SetValue(data['description'])
            if self.tree.append_forms(data['actions']):
                self.enable_actions(True)
            # add it to the history
            self.filehistory.AddFileToHistory(filename)
            self._set_filename(filename)
        
    def _save(self, filename=None):
        self.set_dirty(False)
        if filename:
            self._set_filename(filename)
        data    = {
            'description'   : self.description.GetValue(),
            'actions'       : self.tree.export_forms(),
        }
        common.save_data(self.filename,data)
        if filename:
            # add it to the history
            self.filehistory.AddFileToHistory(filename)
        
    def _set_filename(self,filename):
        self.filename = filename
        self.set_dirty(False)
        
    #---view show/hide
    def show_description(self,checked):
        self.settings['description']    = checked
        self.description.Show(checked)
        self.menu_view.Check(self.menu_view_description.GetId(),checked)
        self.frame_toolbar.ToggleTool(self.toolbar_description.GetId(),checked)
        self.Layout()
        
    def show_droplet(self,checked):
        if checked:
            if base.check_actionlist(self.tree.export_forms()):
                self.droplet = droplet.Frame(self,
                    title       = _("Drag & Drop")+' - '+ct.TITLE,
                    bitmap      = graphics.bitmap(images.DROPLET),
                    method      = self.on_drop,
                    label       = self.droplet_label_format(os.path.splitext(
                                    os.path.basename(self.filename))[0]),
                    label_colour= wx.BLUE,
                    label_pos   = (35, 60),
                    label_angle = 315,
                    pos         = self.GetPosition(),
                    auto        = True,
                    OnShow      = self.on_show_droplet,
                    tooltip     = _(
                    "Drop any files and/or folders on this Phatch droplet\n"
                    "to batch process them.\n"
                    "Right-click or double-click to switch to normal view.")
                )
            else:
                wx.CallAfter(self.on_show_droplet,False)
        else:
            if self.droplet:
                self.droplet.show(False)
            
    def on_show_droplet(self,bool):
        self.menu_view.Check(self.menu_view_droplet.GetId(),bool)
        if not bool: self.droplet = None #don't keep a reference
        
    #---dialogs
    def show_error(self,message):
        return self.show_message(message,style=wx.OK|wx.ICON_ERROR)
            
    def show_execute_dialog(self,result,settings,last_action,files=None):
        dlg = dialogs.ExecuteDialog(self,drop=files)
        if last_action.is_overwrite_existing_images_forced():
            settings['overwrite_existing_images'] = True
            dlg.overwrite_existing_images.Disable()
        if files:
            settings['path']   = files
        dlg.import_settings(settings)
        result['cancel'] = dlg.ShowModal() == wx.ID_CANCEL
        if result['cancel']:
            dlg.Destroy()
            return
        #Retrieve settings from dialog
        dlg.export_settings(settings)
        dlg.Destroy()
        
    def show_files_message(self,result,message,title,files):
        dlg     = dialogs.FilesDialog(self,message,title,files)
        x0, y0  = self.GetSize()
        x1, y1  = dlg.GetSize()
        x       = max(x0,x1)
        y       = max(y1,200)
        dlg.SetSize((x,y))
        result['cancel'] = dlg.ShowModal() == wx.ID_CANCEL
    
    def show_message(self,message,title='',style=wx.OK | wx.ICON_EXCLAMATION):
        dlg = wx.MessageDialog(self, 
                message,
                '%(name)s '%ct.INFO +title,
                style,
        )
        answer  = dlg.ShowModal()
        dlg.Destroy()
        return answer
    
    def show_progress(self,title,maximum):
        dlg = dialogs.ProgressDialog(self,title,_('Starting...'),maximum)
    
    def show_progress_error(self,result,message,details,ignore=True):
        errorDlg    = dialogs.ErrorDialog(self,message,details,ignore)
        answer      = errorDlg.ShowModal()
        result['stop_for_errors']   = not errorDlg.future_errors.GetValue()
        errorDlg.Destroy()
        if answer == wx.ID_ABORT:
            result['answer']    = 'abort'
        elif answer == wx.ID_FORWARD:
            result['answer']    = 'skip'
        else:
            result['answer']    = 'ignore'

    def show_scrolled_message(self,message,title,**keyw):
        import wx.lib.dialogs
        dlg = wx.lib.dialogs.ScrolledMessageDialog(self, message, title, 
                **keyw) 
        dlg.ShowModal()
        
    def setting(self,name):
        return self.settings[name]
    
    #---checks
    def is_last_action_valid(self,last_action):
        if last_action.is_valid_last_action():
            return True
        self.append_save_action()
        return False
        
    def append_save_action(self):
        self.show_error(
            _("There should be a 'Save' action at the end.")+" "+
            _("Phatch will add one for you, please check its settings."))
        self.tree.append_form_by_label_to_last('Save')

    #---other events
    def _events(self):
        #wxPython events
        self.Bind(wx.EVT_CLOSE,self.on_close,self)
        self.Bind(wx.EVT_SIZE,self.on_size,self)
        self.Bind(wx.EVT_MENU_HIGHLIGHT_ALL,self.on_menu_tool_enter)
        self.Bind(wx.EVT_TOOL_ENTER,self.on_menu_tool_enter)
        self.tree.Bind(wx.EVT_TREE_END_DRAG,self.on_tree_end_drag,self.tree)
        self.tree.Bind(wx.EVT_CONTEXT_MENU, self.on_context_menu,self.tree)
        self.logo.Bind(wx.EVT_ERASE_BACKGROUND,self.on_erase_background)
        
    def on_drop(self,filenames,x,y):
        path    = ct.PATH_DELIMITER.join(filenames)
        base.apply_actions(self.tree.export_forms(),self.settings,
            path=path,drop=True)
        
    def on_menu_tool_enter(self,event):
        self.tree.close_popup()
        event.Skip()
        
    def on_tree_end_drag(self,event):
        event.Skip()
        wx.CallAfter(self.set_dirty,True)
        
    def on_erase_background(self,event):
        dc          = event.GetDC()
        if not dc:
            dc      = wx.ClientDC(self.logo)
            rect    = self.logo.GetUpdateRegion().GetBox()
            dc.SetClippingRect(rect)
        dc.Clear() 
        logo        = graphics.bitmap(images.LOGO)
        lw, lh      = logo.GetSize()
        tw, th      = self.logo.GetSize()
        dc.DrawBitmap(logo, (tw-lw)/2, (th-lh)/2, True)
            
    def on_context_menu(self,event):
        if self.tree.is_form_selected():
            self.tree.PopupMenu(self.menu_edit)
        
    def on_close(self,event=None):
        if self.dirty:
            answer = self.show_message(
                _('Before closing, save last changes to')+
                    '\n"%s"?'%self.filename,
                style   = wx.YES_NO|wx.CANCEL|wx.ICON_EXCLAMATION,
            )
            if answer == wx.ID_CANCEL:
                return
            if answer == wx.ID_YES:
                self.on_menu_file_save(None)
        self.Hide() 
        self._saveSettings()
        #Destroy everything
        self.Destroy()
        
    def on_size(self,event):
        event.Skip()
        if self.tree.has_forms():
            self.tree.resize_popup() 
        else:
            self.logo.Refresh() 
        
    #---pubsub
    def _pubsub(self):
        Receiver.__init__(self,'frame')
        #Pubsub events
        self.subscribe('append_save_action')
        self.subscribe('set_dirty')
        self.subscribe('show_execute_dialog')
        self.subscribe('show_error')
        self.subscribe('show_files_message')
        self.subscribe('show_progress')
        self.subscribe('show_progress_error')
        self.subscribe('show_scrolled_message')
        
    #---file history
    def _get_file_history(self):
        result  = [] 
        for index in range(self.filehistory.Count):
            filename = self.filehistory.GetHistoryFile(index)
            if filename.strip():
                result.append(filename)
        return result
    
    def _set_file_history(self,files):
        for filename in files:
            self.filehistory.AddFileToHistory(filename)
 
    #---settings
    def _loadSettings(self):
        self.settings   = base.SETTINGS.copy()
        if os.path.exists(ct.SETTINGS_PATH):
            f               = open(ct.SETTINGS_PATH,'rb')
            self.settings.update(cPickle.load(f))
            f.close()
            
    def _saveSettings(self):
        self.settings['file_history']   = self._get_file_history()
        f   = open(ct.SETTINGS_PATH,'wb')
        cPickle.dump(self.settings,f)
        f.close()
        
    def set_dirty(self,value):
        self.dirty  = ('','*')[value]
        self._title()
        
#---Application
class App(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        #frame
        self.splash = self._splash() 
        self.splash.CentreOnScreen()
        self.SetTopWindow(self.splash)
        self.splash.Show()
        wx.CallAfter(self.show_frame)
        return 1
    
    def _splash(self):
        return droplet.Frame(None,
            title       = ct.TITLE,
            bitmap      = graphics.bitmap(images.DROPLET),
            splash      = True,
        )

    def show_frame(self):
        #get splash
        self.import_modules()
        common.import_actions()
        #do all application initialisation
        base.init()
        #create frame
        frame = Frame(None, -1, ct.TITLE)
        frame.CentreOnScreen()
        frame.Show()
        self.SetTopWindow(frame)
        #delete splash
        self.splash.Hide()
        self.splash.Destroy()
        del self.splash
        
    def import_modules(self):
        global cPickle, gettext, os
        import cPickle, gettext, os
        global base, common
        from core import base, common
        global dialogs
        from pyWx import dialogs
        global ICON_SIZE
        from treeEdit import ICON_SIZE
        

def main(paths):
    app = App(0)
    app.MainLoop()

if __name__ == "__main__":
    main()