"""All translations should be done here. Only controls eg. choice controls
can give 100%safe english strings.
SetValue(english)
Display(dutch) #or any other language
GetValue(english)"""

import os
import wx
import wx.lib.colourselect

####Plain Controls (no i18N)

_t  = unicode
    
#---wildcard
def wildcard_list(name,extensions):
    return '%s|%s|'%(name,
        ';'.join([_wildcard_extension(ext) for ext in extensions]))
        
def _wildcard_extension(ext):
    return '*.%s;*.%s;*.%s'%(ext,ext.upper(),ext.capitalize()) 

#---sizer
def SetMinVerSize(item,size,border=0):
    item_size   = item.GetSize()
    min_size    = (size[0],min(item.GetSize()[1],size[1]-2*border))
    item.SetMinSize(min_size)
    item.SetSize(min_size)
    #debug log
    #sys.stdout.write('SetMinVerSize(border=%s,size=%s,min_size=%s,get_size=%s)\n'\
    #    %(border,size,min_size,item_size))
    
class ForcedBoxSizer(wx.BoxSizer):
    def __init__(self,orient,height,border=0):
        super(ForcedBoxSizer,self).__init__(orient)
        self._size      = (height,height)
        self._border    = border
        
    def AddForced(self,item,proportion,flag=wx.ALL|wx.ALIGN_CENTER_VERTICAL,
            border=None,size=None):
        if not border:  border  = self._border
        if not size:    size    = self._size
        SetMinVerSize(item,size,border)
        super(ForcedBoxSizer,self).Add(item,proportion,flag,border)
        
#---exceptions
class NotImplementedError(Exception):
    
    def __init__(self, instance, method):
        """Not implemented error."""
        self.instance       = instance
        self.method         = method
        
    def __str__(self):
        return 'Class "%s" did not implement method "%s".'\
            %(self.instance.__class__.__name__,self.method)
    
#---base controls
class _Ctrl(object):
    _to_local       = str
    _to_english     = str
    
    def Set(self,value):
        self.SetValue(self._to_local(value))
        
    def Get(self):
        return self._to_english(self.GetValue())
    
class _ComposedCtrl(_Ctrl,wx.Panel):
    """Composed controls are a wx.Panel with controls (extra button,...)."""
    def __init__(self,parent,value,size,**extra):
        super(_ComposedCtrl,self).__init__(parent,-1,size=size)
        self._CreateCtrls(value,**extra)
        self._Layout(height=size[1])
        self._CreateEvents()
        self.SetBackgroundColour(parent.GetBackgroundColour())
        
    def _Layout(self,height):
        sizer = ForcedBoxSizer(wx.HORIZONTAL,height,0)
        self._AddCtrls(sizer,height)
        self.SetSizer(sizer)
        sizer.Fit(self) 
        self.Layout()
        
    #---hooks to be overwritten
    def _CreateCtrls(self):
        """Needs to be overwritten."""
        raise NotImplementedError(self,"_CreateCtrls")
        
    def _AddCtrls(self,sizer,height):
        """Needs to be overwritten."""
        raise NotImplementedError(self,"_AddCtrls")
        
    def _CreateEvents(self):
        """Needs to be overwritten."""
        raise NotImplementedError(self,"_CreateEvents")
        
    def GetValue(self):
        raise NotImplementedError(self,"GetValue")

    def SetValue(self):
        raise NotImplementedError(self,"SetValue")

class _PathCtrl(_ComposedCtrl):
    def _CreateCtrls(self,value,extensions=[]):
        #folder
        self.path = wx.TextCtrl(self, -1, value)
        self.path.SetSelection(-1,-1)
        #browse button
        bmp         = wx.ArtProvider_GetBitmap(wx.ART_FOLDER_OPEN,wx.ART_OTHER,
                        size=(16,16))
        self.browse = wx.BitmapButton(self, -1, bmp, style=wx.NO_BORDER)
        #extensions
        self._extensions = extensions
        
    def _AddCtrls(self,sizer,height):
        if wx.Platform == '__WXGTK__':
            #gtk provides a border already
            b = 0
        else:
            #windows & mac like some distance
            b = 4
        #don't force button, as crop is preferred to being crippled
        sizer.Add(self.browse, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.RIGHT, b)
        sizer.AddForced(self.path, 1)
        
    def _CreateEvents(self):
        self.Bind(wx.EVT_BUTTON,self.OnBrowse,self.browse)
        
    def GetDefaultPath(self):
        default_path     = os.path.dirname(self.path.GetValue())
        old             = None
        while not os.path.exists(default_path) and (default_path!=old):
            old         = default_path
            default_path = os.path.dirname(default_path)
        return default_path

    def GetValue(self):
        return self.path.GetValue()
    
    def SetBackgroundColour(self,colour):
        super(_PathCtrl,self).SetBackgroundColour(colour)
        self.browse.SetBackgroundColour(colour)
        
    def SetValue(self,x):
        self.path.SetValue(x)
        
    def SetFocus(self):
        self.path.SetFocus()
        
METRICS     = ['px','%','cm','mm','inch']
METRICS_MAX = max([len(x) for x in METRICS])
#---Main Controls (will be automatically registered through meta-class)
class BooleanCtrl(_Ctrl,wx.CheckBox):
    def __init__(self, parent, value, size):
        super(BooleanCtrl,self).__init__(parent,-1,'',size=size)
        self.SetValue(bool(value))
        
    def Get(self):
        return (_t('no'),_t('yes'))[self.GetValue()]
    
class ChoiceCtrl(_Ctrl,wx.Choice):
    def __init__(self,parent,value,size,choices):
        local_choices  = [self._to_local(choice) for choice in choices]
        super(ChoiceCtrl,self).__init__(parent,-1,choices=local_choices,
            size=size)
        self._choices       = choices
        self.Set(value)
        
    def Get(self):
        index       = super(ChoiceCtrl,self).GetSelection()
        if index == wx.NOT_FOUND:
            index   = 0
        return self._choices[index]
    
    def SetValue(self,value):
        self.SetStringSelection(value)
    
def HTMLColorToRGB(colorstring):
    """ convert #RRGGBB to an (R, G, B) tuple """
    colorstring = colorstring.strip()
    if colorstring[0] == '#': colorstring = colorstring[1:]
    if len(colorstring) != 6:
        raise ValueError, "input #%s is not in #RRGGBB format" % colorstring
    r, g, b = colorstring[:2], colorstring[2:4], colorstring[4:]
    r, g, b = [int(n, 16) for n in (r, g, b)]
    return (r, g, b)

class ColourCtrl(_Ctrl,wx.lib.colourselect.ColourSelect):
    def __init__(self, parent, value, size):
        label       = value
        if isinstance(value, (str, unicode)):
            value   = HTMLColorToRGB(value)
        super(ColourCtrl,self).__init__(parent,-1,'',value,size=size)
        self.Bind(wx.lib.colourselect.EVT_COLOURSELECT, self.OnSelectColour)
        wx.CallAfter(self.SetLabel,label)
        wx.CallAfter(self.SetValue,value)
        
    def GetValue(self):
        return self.GetColourAsString()
    
    def GetColourAsString(self,colour=None):
        if colour == None:
            colour  = self.GetColour()
        if isinstance(colour, (str, unicode)):
            return colour
        return colour.GetAsString(wx.C2S_HTML_SYNTAX)
    
    def OnSelectColour(self,event):
        colour  = event.GetValue()
        self.SetLabel(self.GetColourAsString(colour))
        self.SetValue(wx.NamedColour(colour))
                    
class FileCtrl(_PathCtrl):
    wildcard  = _t('All files')+'|*'
    
    def OnBrowse(self,event):
        dlg = wx.FileDialog(self, self._to_local("Choose a file"),
            defaultDir      = self.GetDefaultPath(),
            wildcard        = self.GetWildcard(),
            style           = wx.OPEN | wx.CHANGE_DIR |wx.FD_PREVIEW
            )
        if dlg.ShowModal() == wx.ID_OK:
            self.path.SetValue(dlg.GetPath())
        dlg.Destroy()
        
    def GetWildcard(self):
        return self._to_local(self.wildcard)
        
class FilePathCtrl(_PathCtrl):
    def OnBrowse(self,event):
        dlg = wx.DirDialog(self,self._to_local("Choose a folder"),
            defaultPath = self.GetDefaultPath(),
            style       = wx.DEFAULT_DIALOG_STYLE)
        if dlg.ShowModal() == wx.ID_OK:
            self.path.SetValue(dlg.GetPath())
        dlg.Destroy()
        
class ImageReadFileCtrl(FileCtrl):
    _images     = _t('Images')
    _all_files  = _t('All files')
    
    def GetWildcard(self):
        return '|'.join([
            wildcard_list(self._to_local(self._images),self._extensions),
            self._all_files+'|*'])
            
class PixelCtrl(_ComposedCtrl):
    def _CreateCtrls(self,value):
        self.size   = wx.TextCtrl(self,-1,value)
        self.size.SetSelection(-1,-1)
        self.unit   = wx.Choice(self,-1,choices=METRICS)
        self.SetValue(value)
        
    def _AddCtrls(self,sizer,height):
        sizer.AddForced(self.size, 1)
        width   = self.unit.GetSize()[0]
        if wx.Platform == '__WXGTK__':
            width = int(width*0.8)
        sizer.AddForced(self.unit, 0,size=(width,height))
        
    def _CreateEvents(self):
        """Nothing to do"""
        
    def SetValue(self,value):
        #split value and unit
        value   = value.strip()
        unit    = METRICS[0]
        for n in range(1,METRICS_MAX+1):
            if len(value)>n and value[-n:] in METRICS:
                unit    = value[-n:]
                value   = value[:-n].strip()
                break
        self.size.SetValue(value)
        self.unit.SetStringSelection(unit)
        
    def GetValue(self):
        value   = self.size.GetValue()
        if value:
            return '%s %s'%(value,self.unit.GetStringSelection())
        else:
            return ''
        
    def SetFocus(self):
        self.size.SetFocus()
    
class SliderCtrl(_ComposedCtrl):
    """Needs to mimic a wx.SliderCtrl"""
    def _CreateCtrls(self,value,minValue,maxValue):
        value       = int(value)
        #spin ctrl
        self.spin   = wx.SpinCtrl(self, -1)
        self.spin.SetRange(minValue, maxValue)
        self.spin.SetValue(value)
        #slider
        self.slider = wx.Slider(self, -1, value , minValue, maxValue, 
                        style = wx.SL_HORIZONTAL)
        
    def _AddCtrls(self,sizer,height):
        sizer.AddForced(self.spin, 0, size=(int(height*1.85),height))
        sizer.AddForced(self.slider, 1)

    def _CreateEvents(self):
        self.Bind(wx.EVT_SPINCTRL,self.OnSpin,self.spin)
        self.Bind(wx.EVT_SCROLL,self.OnScroll,self.slider)
        
    #---control methods (obligatory)
    def GetValue(self):
        return unicode(self.slider.GetValue())
    
    def SetBackgroundColour(self,colour):
        super(SliderCtrl,self).SetBackgroundColour(colour)
        if wx.Platform == '__WXMSW__':
            self.spin.SetBackgroundColour(colour)
        self.slider.SetBackgroundColour(colour)
        
    def SetFocus(self):
        self.spin.SetFocus()
            
    #---events
    def OnSpin(self,event):
        self.slider.SetValue(self.spin.GetValue())
        
    def OnScroll(self,event):
        self.spin.SetValue(self.slider.GetValue())
        
#todo implement all controls as panel with custom BoxSizer
#or maybe PopupControl should take care of this
class TextCtrl(_Ctrl,wx.TextCtrl):
    def __init__(self,parent,value,size):
        super(TextCtrl,self).__init__(parent,-1,value,size=size)
        self.SetSelection(-1,-1)
    
CTRL_CACHE          = {}

def ctrl_factory(name,CtrlMixin):
    ctrl_key                = (name,CtrlMixin)
    try:
        Ctrl                = CTRL_CACHE[ctrl_key]
    except:
        Ctrl                = globals().get(name+'Ctrl',TextCtrl)
        if not (CtrlMixin is None):
            if isinstance(CtrlMixin,list):
                bases       = tuple(CtrlMixin+[Ctrl])
            else:
                bases       = (CtrlMixin,Ctrl)
            Ctrl            = type(name,bases,{})
        CTRL_CACHE[ctrl_key]= Ctrl
    return Ctrl

class EditPanel(wx.Panel):
    def __init__(self,parent,typ,value, extra={},size=(28,28),pos=(0,0),
            offset=0,label='',border=0, CtrlMixin=None):
        super(EditPanel,self).__init__(parent,-1,pos=pos,size=size)
        self._SetColours()
        if label: self._CreateLabel(label)
        height  = size[1] 
        self._CreateEdit(value,extra,typ,height,border,CtrlMixin) 
        self._Layout(offset,height,border,label)
        
    def _CreateLabel(self,label):
        self.label      = label
        self.labelCtrl  = wx.StaticText(self,-1, label)
        self.labelCtrl.SetForegroundColour(self.fgcolour)
        #print '_CreateLabel',self.fgcolour
            
    def _CreateEdit(self,value_as_string,extra,typ,height,border,CtrlMixin):
        #create ctrl class
        Ctrl            = ctrl_factory(typ,CtrlMixin)
        #create ctrl instance
        self.edit       = Ctrl(self,value=value_as_string,
                            size=(height,height-2*border),**extra)
        #check min size
        if self.edit.GetSize()[1] > height:
            self.edit.SetMinSize((self.edit.GetMinSize()[0],height))
        self.edit.SetFocus()
        
    def _Layout(self,offset,height,border,label):
        sizer = ForcedBoxSizer(wx.HORIZONTAL,height,border)
        sizer.AddSpacer(offset)
        if label:
            sizer.Add(self.labelCtrl, proportion = 0, 
                flag = wx.ALL|wx.ALIGN_CENTER_VERTICAL)
        sizer.AddForced(self.edit, proportion = 1,border=border)
        if wx.Platform != '__WXGTK__':
            sizer.AddSpacer(min(4,border))
        self.SetSizer(sizer)
        sizer.Fit(self)
        self.Layout()
        
    def Close(self):
        """Loses focus"""
        if self:
            result  = unicode(self.edit.Get())
            self.Destroy()
            return result
            
    #---support methods
    def _SetColours(self):
        self.bgcolour  = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT)
        self.fgcolour= wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT)
        self.SetBackgroundColour(self.bgcolour)

def test():
    width, height   = 300, 28
    obj             = globals()
    ctrls           = [(name[:-4],obj[name]) for name in globals().keys()
                        if name.endswith('Ctrl') and not name.startswith('_')]
    ctrls.sort()
    class App(wx.App):
        def OnInit(self,*args,**keyw):
            frame       = wx.Frame(None,-1,'popup test')
            sizer       = wx.BoxSizer(wx.VERTICAL)
            for typ, ctrl in ctrls:
                if issubclass(ctrl,SliderCtrl):
                    extra   = {'minValue' : 0,'maxValue' : 100}
                elif issubclass(ctrl,ChoiceCtrl):
                    extra   = {'choices' : ('1','2')}
                elif issubclass(ctrl,FileCtrl):
                    extra   = {'extensions' : ('*.py','*.png')}
                else:
                    extra   = {}
                if issubclass(ctrl,ColourCtrl):
                    value   = '#FFFFFF'
                else:
                    value   = '1'
                popup       = EditPanel(frame,typ,value=value,extra=extra,
                                size=(width,height),pos=(0,0),offset=10,
                                label=typ, border=1,CtrlMixin=None)
                sizer.Add(popup,flag=wx.EXPAND)
                
            frame.SetSizer(sizer)
            sizer.Fit(frame)
            
            frame.SetSize((width,frame.GetSize()[1]))
            frame.Layout()
            frame.Show()
            self.SetTopWindow(frame)
            return True
        
    app = App(0)
    app.MainLoop()
    
if __name__ == '__main__':
    test()
