# License: latest GPL

#---import modules

# gui-indepedent
if __name__ == '__main__':
    import sys
    sys.path.insert(0,'../../core/lib')

import formField

# gui-dependent
import wx
import graphics, popup, treeDragDrop
                            
FIELD_DELIMITER = ': '
IMAGE_TEST_INFO = {'size': (1, 1), 'format': None, 'height': 1, 'width': 1, 
    'mode': 'L', 'folder': '/test', 'filename': 'unknown', 'type': 'png', 
    'dpi': 72, 'location': '/test/unknown.png'}
    #generated from pil.create_test_info()

#---functions

#---tree
ITEM_HEIGHT = 28
ICON_SIZE   = (ITEM_HEIGHT,ITEM_HEIGHT)
TR_DEFAULT_STYLE   = wx.TR_HAS_BUTTONS|wx.TR_NO_LINES|wx.TR_FULL_ROW_HIGHLIGHT|wx.TR_HIDE_ROOT|wx.TR_DEFAULT_STYLE|wx.SUNKEN_BORDER

def _do_nothing(*args,**keyw):
    pass

def rescale(image,x,y,filter=None):
    """For compatibility with wxPython 2.6"""
    if filter is None and hasattr(wx,'IMAGE_QUALITY_HIGH'):
        filter  = wx.IMAGE_QUALITY_HIGH
    if filter is None:
        image.Rescale(x,y)
    else:
        image.Rescale(x,y,filter)

class TreeMixin(treeDragDrop.Mixin):
    def __init__(self,form_factory={},CtrlMixin=[],
        icon_size=(28,28),show_error=_do_nothing,set_dirty=_do_nothing):
        if wx.Platform == "__WXGTK__":
            #indentation
            self.SetIndent(int(self.GetIndent()*1.5))
        #form factory
        self.form_factory       = form_factory 
        self.CtrlMixin          = CtrlMixin
        #popup
        self.popup              = None
        self.popup_item         = None
        self._field_selected    = False
        #methods
        self.show_error         = show_error
        self.set_dirty          = set_dirty
        #image list
        self.CreateImageList(icon_size)
        #drag & drop
        self.EnableDrag(dragTo=self.get_form_item)
        #clear
        self.delete_all_forms()
        #events
        self.events()
        
    def CreateImageList(self,icon_size):
        self.image_list = wx.ImageList(*icon_size)
        icon_disabled   = graphics.bitmap(ICON_DISABLED)
        for form in self.form_factory.values():
            self._AddFormToImageList(form,icon_size,icon_disabled)
        self.SetImageList(self.image_list)
        
    def _AddFormToImageList(self,form,icon_size,icon_disabled):
        image             = graphics.image(form.icon,icon_size)
        form.icon_bitmap  = wx.BitmapFromImage(image)
        rescale(image,icon_size[0],icon_size[1])
        form.icon_tree    = wx.BitmapFromImage(image)
        form.icon_tree_disabled   = icon_disabled
        form.icon_tree_id         = (
            self.image_list.Add(form.icon_tree_disabled),
            self.image_list.Add(form.icon_tree))
            
    def set_item_image(self,x,image):
        self.SetItemImage(x,image,wx.TreeItemIcon_Normal)
        
    def treeLabel(self,name,value):
        return ''.join([_(name),FIELD_DELIMITER,
            self.CtrlMixin._to_local(value)]) 
            
    def delete_all_forms(self):
        self.DeleteAllItems()
        self.AddRoot('')
        
    #---forms
    def append_form(self, form, item=-1):
        root    = self.GetRootItem()
        if item is -1:
            item    = self.AppendItem(root,_(form.label))
        else:
            item    = self.InsertItem(root,item,_(form.label))
        self.set_item_image(item,form.icon_tree_id[True])
        self.SetItemBold(item,True)
        self.import_form(item,form)
        self.SelectItem(item)
        return item
        
    def append_forms(self,forms):
        for form in forms:
            self.append_form(form)
        return forms
        
    def append_form_by_label(self,item,label):
        return self.append_form(self.form_factory[label](),item)
        
    def collapse_forms(self):
        root    = self.GetRootItem()
        for child in self.GetItemChildren(root):
            self.Collapse(child)
        
    def enable_form(self,item, bool):
        self.enable_form_item(self.get_form_item(item),bool)
        
    def enable_form_item(self,item,bool):
        form              = self.GetPyData(item)
        self.set_item_image(item,form.icon_tree_id[bool])
        self.SetItemTextColour(item,(wx.RED,wx.GREEN)[bool])
        
    def expand_forms(self):
        root    = self.GetRootItem()
        for child in self.GetItemChildren(root):
            self.Expand(child)
                      
    def export_form(self,item,label=None):
        form              = self.GetPyData(item)
        for field in self.GetItemChildren(item):
            label, value_as_string  = self.GetPyData(field)
            form.set_field_as_string(label,value_as_string)
        form.set_field('__enabled__',self.is_form_enabled(item))
        return form

    def export_forms(self):
        root    = self.GetRootItem()
        forms = []
        for child in self.GetItemChildren(root):
            forms.append(self.export_form(child))
        return forms

    def import_form(self,item,form):
        self.SetPyData(item,form)
        self.DeleteChildren(item)
        fields = form.get_fields()
        for label, field in fields.items():
            if field.visible:
                value_as_string = field.get_as_string()
                oItem           = self.AppendItem(item,
                    self.treeLabel(label,value_as_string))
                self.SetPyData(oItem,(label,value_as_string))
            elif label == '__enabled__':
                self.enable_form_item(item,field.get())
        self.Expand(item)
        
    def get_form(self,item,label=None):
        return self.export_form(self.get_form_item(item),label)
    
    def get_form_item(self,item):
        return self.GetRootChild(item)
    
    def get_form_field(self,item):
        label, value_as_string  = self.GetPyData(item)
        return self.get_form(item,label).get_field(label)
    
    def has_forms(self):
        return self.GetCount()
    
    def toggle_form_item(self,item):
        root        = self.GetRootItem()
        if item == root:
            event.Skip()
        else:
            parent  = self.GetItemParent(item)
            if parent == root:
                image   =  self.GetItemImage(item,wx.TreeItemIcon_Normal)
                if image != -1:
                    self.enable_form_item(item,
                        not self.is_form_enabled(item))
                    
    def set_form_field_value(self,item, value_as_string):
        label, old  = self.GetPyData(item)
        if value_as_string.strip() and value_as_string != old:
            form      = self.get_form(item,label)
            field       = form.get_field(label)
            #validate
            try:
                if isinstance(field,formField.PixelField):
                    field.get_size(IMAGE_TEST_INFO, 100, 100,value_as_string)
                    #100 is just some dummy value for base, dpi
                else:
                    field.get(IMAGE_TEST_INFO, value_as_string)
                self.set_dirty(True)
            except formField.ValidationError, error:
                self.show_error(error.message)
                value_as_string   = old
            self.SetPyData(item,(label,value_as_string))
            self.SetItemText(item,self.treeLabel(label,value_as_string))

    #---selected form
    def append_form_by_label_to_selected(self,label):
        item    = self.get_selected_form() 
        return self.append_form_by_label(item,label)

    def enable_selected_form(self, bool):
        self.enable_form_item(self.get_selected_form(),bool)
        
    def get_selected_form(self):
        if self.has_forms():
            return self.get_form_item(self.GetSelection())
        return -1
    
    def move_selected_form_down(self):
        self.MoveChildDown(self.get_selected_form())
        
    def move_selected_form_up(self):
        self.MoveChildUp(self.get_selected_form())
        
    def remove_selected_form(self):
        form  = self.get_selected_form()
        if form is -1:
            return False
        else:
            self.Delete(form)
            return True
        
    #---last form
    def append_form_by_label_to_last(self,label):
        item    = self.get_last_form()
        self.append_form_by_label(item,label)

    def get_last_form(self):
        return self.GetLastChild(self.GetRootItem())
            
    #---popup
    def create_popup(self,item):
        #self.SelectItem(item,False)
        field               = self.get_form_field(item)
        label, value        = self.GetItemText(item).split(FIELD_DELIMITER)
        pos, offset, size   = self.get_popup_pos_offset_size(item)
        typ                 = field.__class__.__name__.replace('Field','')
        if isinstance(field,formField.SliderField):
            extra   = {'minValue' : field.min,'maxValue' : field.max}
        elif isinstance(field,formField.ChoiceField):
            extra   = {'choices' : field.choices}
            typ     = 'Choice'
        elif isinstance(field,formField.FileField):
            extra   = {'extensions' : field.extensions}
        else:
            extra   = {}
        self.popup          = popup.EditPanel(self, pos = pos, offset = offset,
                                size = size, label = _(label)+FIELD_DELIMITER, 
                                value = value, extra = extra, typ = typ,
                                border = 1, CtrlMixin   = self.CtrlMixin)
        self.popup_item     = item
        self.popup.Show()
        self.resize_popup() 
        
    def create_popup_selected(self):
        item    = self.GetSelection()
        if self.GetItemParent(item) == self.GetRootItem():
            return
        self.create_popup(item)
        
    def close_popup(self):
        if not(self.popup is None):
            value_as_string = self.popup.Close()
            self.set_form_field_value(self.popup_item, value_as_string)
            self.popup  = self.popup_item = None
            
    def get_popup_pos_offset_size(self, item):
        text_only_rect  = self.GetBoundingRect(item,textOnly=True)
        rect            = self.GetBoundingRect(item,textOnly=False)
        pos             = rect.GetPosition()
        size            = rect.GetSize()
        offset          = text_only_rect.GetPosition()[0] - pos[0]
        return pos, offset, size
        
    def resize_popup(self):
        if not(self.popup is None):
            item                = self.GetSelection()
            pos, offset, size   = self.get_popup_pos_offset_size(item)
            self.popup.SetSize(size)
            self.popup.SetPosition(pos)
            self.popup.Layout() 
        
    #---events
    def events(self):
        #events
        self.Bind(wx.EVT_TREE_SEL_CHANGING, self.on_sel_changing, self)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.on_sel_changed, self)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.on_item_activated, self)
        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK,self.on_select,self)
        self.Bind(wx.EVT_LEFT_DOWN,self.on_left_down,self)
        
    def on_left_down(self,event):
        self.close_popup()
        event.Skip()

    def on_item_activated(self,event):
        item        = event.GetItem()
        if self.is_form(item):
            self.toggle_form_item(item)
        else:
            self.on_sel_changed(event)
        
    def on_sel_changing(self,event):
        self.close_popup()
        
    def on_sel_changed(self,event):
        self.close_popup()
        item                    = event.GetItem()
        self._field_selected    = self.is_field(item)
        if self._field_selected:
            self.create_popup(item)
        
    def on_select(self,event):
        self.on_sel_changing(event)
        self.SelectItem(event.GetItem(),True)
        event.Skip()
        
    #---checks
    def is_field(self,item):
        return self.GetItemParent(item)!= self.GetRootItem() and \
                item != self.GetRootItem()
    
    def is_field_selected(self):
        return self._field_selected
        
    def is_form(self,item):
        return self.GetItemParent(item)== self.GetRootItem()
        
    def is_form_enabled(self,item): 
        form              = self.GetPyData(item)
        return self.GetItemImage(item,wx.TreeItemIcon_Normal)==form.icon_tree_id[True]
        
    def is_form_selected(self):
        return not self._field_selected
        
def test():
    global _
    _ = str
    import sys
    
    class Form1(formField.Form):
        label   = 'form1'
        def __init__(self):
            formField.Form.__init__(self,
                fields = {
                    'foo1':formField.ChoiceField(value='a',choices=('a','b')),
                })
    
    class Form2(formField.Form):
        label   = 'form2'
        def __init__(self):
            formField.Form.__init__(self,
                fields   = {'foo2':formField.PixelField(value='100'),})
    
    form_factory    = {
        'form1' : Form1,
        'form2' : Form2,
    }
    form3   = formField.Form(
                fields   = {'foo3':formField.SliderField(value='100',
                    minValue=0,maxValue=100),})
    forms   = [x() for x in form_factory.values()] #+ [form3]
    
    class Tree(wx.TreeCtrl,TreeMixin):
        
        def __init__(self,parent,form_factory,*args,**keyw):
            
            class I18n_CtrlMixin:
                """Fake example of a Mixin"""
                _to_local   = str
                _to_english = str
                _to_local   = staticmethod(_to_local)
                _to_english = staticmethod(_to_english)
                
            wx.TreeCtrl.__init__(self,parent,style=TR_DEFAULT_STYLE,*args,**keyw)
            TreeMixin.__init__(self,
                form_factory= form_factory,
                CtrlMixin   = I18n_CtrlMixin,
                icon_size   = (28,28),
                show_error  = parent.show_error,
                set_dirty   = parent.set_dirty,
            )
            
    class Frame(wx.Frame):
        def show_error(self,message):
            sys.stdout.write(message+'\n')
            
        def set_dirty(self,bool):
            self.SetTitle(['','*'][bool]+'treeEdit test')
            
    class App(wx.App):
        def OnInit(self,*args,**keyw):
            frame       = Frame(None)
            frame.set_dirty(False)
            
            sizer       = wx.BoxSizer(wx.VERTICAL)
            self.tree   = Tree(frame,form_factory)
            self.tree.append_forms(forms)
            sizer.Add(self.tree,proportion=1, flag=wx.EXPAND)
            frame.SetSizer(sizer)
            sizer.Fit(frame)
            
            frame.SetSize((300,300))
            frame.Layout()
            frame.Show()
            self.SetTopWindow(frame)
            return True
        
        
    app = App(0)
    app.MainLoop()
        
#---disabled
ICON_DISABLED = \
'x\xda\x01M\x07\xb2\xf8\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x1c\
\x00\x00\x00\x1c\x08\x06\x00\x00\x00r\r\xdf\x94\x00\x00\x00\x04sBIT\x08\x08\
\x08\x08|\x08d\x88\x00\x00\x07\x04IDATH\x89\x85\x96\xcb\xafeE\x15\x87\xbfz\
\xec\xc7y\x9f\xbe}\xfb\xde\xbet\xdb\rH\xd2m@\x13\x10\x8c&$\x1a\xe2s\x80&\x06\
\x19(\x13EI\x88\t\xfe\x01\xc6\x81\x13\xa6\x1a\x07:\x91!\x13\x8d1\n\x13\xa3\
\x88\x1dM4D\xd2\n"\xca#\xc1\xee\xdb\x0f\xee\xed\xd3\xa7\xef\xd9\xfb\x9c\xfd\
\xa8\xc7r\xb0O#\x08\xad;Y\xa9]\xa9\xda\xeb\xdbU\xb5~\xab\x96\xe2\x7f<\xf2\
\xd8\x00\x86\xd3\x84\xf1\xc6\x10-\x87h\x9a\x9bQ\xea8Q\x1d\x02\xf1\xc4v\x8f \
\xe7@\xed\xd2\xb8\x03\x0e\xca\x8a\x7f\x1c\x04u\xe6\xc6>\xed\ra\x0f\xa1\x99n\
\xdfL\x9a~\x0c\xa5?G>\xb8\x87Qv\x1c\x93\xf5\xd0F!\x02m\x15\xa9W\x0b\xaa\xf2\
\x15|q\x86~\xf6+\xee\x18\xbf\xc0\x99\xc5\xfe\x8d\xfc\xaa\xf7\x84}\xb3?f\xba}\
/\xd6<Bo\xfc\t\x06\x1b\x13\xc6Ga\xb2\x03\xe3MH\xfb\xd08h\x96\xb0\x9c\xc1\xb5\
]\x98\x9d\x87\xf9\xa5s,\x17\xbf\xa0i\x9f\xe0\xd2\xeao\xeai\xc2\xff\x05\xca\
\xa3\xfdm\x06\x93\x870\xf6\x1b\xf4\xa7\xa7\xd8\xf9\x00\x9c\xb8\x03\xb6O\xc0\
\xf80\x0cFP9\xb8:\x07\xe7\xa0]A\xb5\x80\x83\x8bp\xe1%8\xffb\xe4`\xf6\x0cu\
\xfb}\xda\xf8\x8c\xfa)\xcd\r\xb7T\x1e6[\xa8\xf4Q\xea\xe6\x11\xf2\xf4(\x87O\
\xc1\xce)\xb8\xe56\x18M\xc0\xe6`4\x90\xc2\xc6\xa4\x03\xfa\x1c\x9a\x1eL\xc60\
\x98\x80N5/={\x9fH3\xf5\x8a\xfc\xcaW\xf8\xe5\xe6\x93\xf8\xeb\x0c\xf3\x16\xec\
\xab\xf4\xd1\xd9\x97\xf1\xf11\x82\xdc\xc4\xb1;\x14\xd3\x9dn0\x06h\x1aX\x16\
\xb0\xb8\x06\xc5\x1cVE\xb7\xa5F\xc1\xe1)\x10`>Gt\x8ejWZ\xe6\x97\xb7\x83\x97\
\xe3\xc6\xf3\xcf\xc7_\xe6\xdc;V(_B\xd1\xa8\x0f\x8bv\x0f\x13\xe4\x18[\'\x95\
\x1a\x1c\x82\xd0@\xab`\xff\x02\x143\xc82H\x120\x16\x94\x81\x18\xa1\xd4`n\xea\
~f9G\\\x03\x9b\xb7\xa2.\xben\xa9\xe7\x1f\r\x91\xaf\x17\x0f\xf2\xaf\xd1O8\x0f\
\xa0\x9f\xfd8\xb4\x96-\x17\xe4!\xd7\x84\xbbZ\xb4\n\xc3\rD\x1cH\x0b\xb4\xa0k\
\xb0\x15$\xab\xb5\x95`\x17\x90\x14\x10\xe7p\xb0\x0b}\x0b=\x83\x8e\x15Q\x1b\
\xe2\xe8\x08(\x12\x1f\xb8\xbfZ\xf2\xa9\xd9\x17I\x01\xf4\x87\x8e\xa0\xdb\x96S\
\xad\xe3\xd3m\x00\xd1\x16\x94\x82\xd0\x02\r\xa8\x1a\xcc\x1ahW`\xcb\x0e\x98\
\x94`\x97`J\xa8\xde\x04Y\xc2t\x004\xa8\xd8\x12M\x82(@\xd8\x08\x91\xcf\xb8\
\x9am\x00\x9b+R\x1f\xb8;\x04N\x18\r\x82\xa0b\x0b\xd2tA\xac\x05L\x04\x1b \xf1\
\x90\x9a\xee\xdcP\x10\x80\x10\xa1\x8eP]\x81\xe1\x00\xac@\xa8\x91\xe0\x88\xd2\
\x9d\x9b\xf7\xdc-\x91\xe3\xc5\xfd\xec\xda\xa6\xa1\x17\x85;cD+\rD\x0f~\t2\\\
\x0b\'\x82\t`=\xa4\xee\xbf\x80\x02"\xe0\x0243\x18\xf7\xa0\x9f\xa0\xf6\x97\
\xd0\x94\x08 \nB\xe4\x98\xd6\x9cn4\xcf\xdbUKn\xe0\xb4\xa8.d\x95\x04X]E\xb9\
\x11H\x04\xe5\xc1$`-$\x16R\rv-_G\x07\xd5\x1e\xa2\x05*\x18\xe7P\xcdQ\xd5\x1c\
\x01\xa2(B\x94,\xc0\xe9UCj\x83\xa3\xef\xe1\x88I4\x11\x10\x15\xa1^ \xab}\xd4t\
\xb3\xf3\xaa,h\x0bZw\xab3\xaa\x8bRQ\x1d<7\xdd\x9c\xb0\x82\x0cX^B|\x8d(\x85\
\xa0\t1\x82\xc8\x0e`m\x14\xd2\xd6\x91eF#J#\xca\x81\x04\xd4\xf22Tt\x82\x0f\
\x1a\xa2\x06\x95B2\x82l\x08\xba\xd7\xc9$\xb3\x90khL\'\x8df\xd6\x05\x9a\x08\
\x11M\xc0\x12\xc4C\x0c\x03\x14\xda:\xc1\x07\xc1\xbbh\x88*A$\x80\x04\xf0\x01\
\x16\x17\xa1W\xc2d\x0b\xf2\x1d\x18\x1c\x82\xbc\xdfm1t\xe7\'\x0eB\x80\xaa\x82\
\xc5\x02V\x0e\x19\x8c\x10Y\x101\x04\x95\xe0B\x80H\x83Bl\xdbR\x01s\xe7\xe5\
\xb8S\x16\x8f%\xc6\x80\x0e\xa0\xa2\x06\x9bB\xd6\xeb\x9c\xd7\x05\xc8\n\xac^G\
\x83@\x13`\xd9B\xd1B\x15\xa1V\x90\xf6\x08h\x82\xcah\xc5\xd0\xb4\x11\xad\xd8\
\xd3\x06o\x8de\xe5=\xaf6M\xf8`\xe6\r>\xef\xe1\x83\xc3\xf8\x88\xca\'\xd0\xdb\
\x80r\x05~\x0e\xbd\x00\xb9t@e!\x18p\x16\xea\x04V\x06*\r\xb5"j\x8b\xb7=Z\xd7\
\xa7ij\x1a\'>I\xd5\xab\xc3\\Z\xdb\xebQ\x15\x05\xcf\xb5M\xfcB\xbd\xf2&\x1f\
\x8dI\xc4\x91(\x87R9\xea\xd2\x1e$\x15\x0c\x1c\x0c\x02\xe4\xeb\x84\xa8\x148\r\
M\x02\xd5\xda\xda\x9c\xe8sB\x9b\xe0\x86\xdb4\xcb\x96\xe5\xb2\xa0\x89\xfaB\
\x08\xbc\xd8\x1f\x05g\x8d\xa5\xa9\x1a\x9e\xf3A.WE},;4%\x1d\x1d&\xd1\x0b\xd4\
\x9bW1\xb6\x81\xa1\xacE\xde%\x9f.\xe5_\xd7_\x80e\r\x05\xc4\xd2\xe0W\tmv\x84\
\xd6nQ\xadv)\x97\x9eV\xcc\xd9\x108\xf7\xbe\x9f\x87\xa8\x7f\xfc4\xd1$\xbc\x86\
V\xbf_U\x9eb^\xb2bDc\x8e\xe0\x8a@X\n,\x81\x05p\x00\\[\xdb|\xdd?`\r\x03_\x04\
\xda\xdaR5C\x96\xfb%\xc5\xbc\xa0t\xba\x0c\xca\x9e\xd9\x98\xc6=\x00\xfd] \x1f\
s\xa9\xd5\xe6gU\xd4\xe7\xe7\xb3\x92\xf9~A\x11\xc6\xd4\xfd[pq\x8c[h\xc25\x90\
\xab 3`\x06\xac\xdf\xe3\xd5\xeex]ii\xcc\x16\xd5\xe8V\x8a6c~a\x9f+\x07\x91\
\x9a\xf4\x8fy\x8fgn:\xe2\xab\xb7\xee\xc3\x1f\xbeN\xfc\xce]q\xf7J\x91f+\xa7\
\xeel\x96U\x1eE\xa3F\x9b0\xd8D\xec\x10\x82E\x1cH\xa3\x88\xb5&\xd6\x86\xd0\
\xa4x7\xc4\xe9#\xd4\xf9\tJ\xb3\xcdA\t{\xaf\xef\xb27\xab\xe2A\xc8^\x8e\xe8\
\xc7\xcf\xfe\xa5\xf8\xc3\xe7\xff\x8e\xbc\xab\xc48{_z\xf3k\x97\xd3o\x8b\xa8\
\x07{\xa9\x8c\xa7;\x9b\x1c:\xba\xc9`\x9c\x91g\x90(\x87\xa1EK@PD\x95\x10$\xa5\
\t\x86U\x15(\xe7%\xd7.\xee1\x9b-\xe32\xa4od\xa9|\xef#\'\xe7O\xbc\xff)\xaa\
\xeb\x8cw\xd54\xbf\xbegp\xdb\xbc2\xdfrA?\x90\x1a9:\x18\xe7\x0c7&\xf4\'#\xf2~\
F\x92\x18\x94\xee\xe6\x86 \xb4\x8d\xa7.+\xca\xf9\x82\xf2\xea\x82e\x15)\xa3}a\
\x98\xc5\x1f\xddu\xa2|\xf2\xf4SM\xf1v\xff\xefY\xb5=\xff\xc9l\xe7\xcf\xe7\xfb\
\x0f\x04\xd4\xd7z\x8a\xdb\xf3D%i\x96\x90\xe6\t6MP\xba\xfb,\xf8\x80k\x1cm\xed\
\xa9\x9b\xc0\xc2\xab\xc5*\xe8?MR\xf9\xc1\xc9\x9d\xbd\xdf|\xf6\xb7\xef,\xa0\
\xae\x03\x15\xb0\xae\x8c\xc8\xd6\xd6\x9b&\x8c\xef\x9d\xf4\xef=\x96&\xf7m\xa5\
\xf6\xf6\x8dD\x1f\xebi5H\xb4\xd2Z\x81\x00\xbe\xbb\n}\xe9\x99\xef\xfb\xf0\xc6\
\xc5\xd6\x9f}eY\xff\xee\xafE\xfdR\xec2q\xcd\x7f\xda\x16\xf0\xd7\x81=`\x04\
\x8c\x81\xe9\xba\x9dh\xe8\x8d\xac\xee\xef\xa4\xc9\xc6vj\xb76\xac=\xda3zj\x94\
\xea\t\xc46R\x16!\xccf\xce_\xbe\xdc\xba\xfd\xfd\xd6-\xea(n\xad\xdab-\xa0\x83\
\xb7\x89\xaa\xbc\xbe\xa5\x9a.\x7f\xa4t\xb9\xa4\x07\xf4\x81\xe1\xba\x9f\x1aE\
\x9a*m\x13\xad\x8c\xe9\xea\x00\xbc \xadH\xf41\xfa\xd8\xdd\x8en\xbd\x92\xe5\
\xdaV\xeb\x156\xeb\xb1\xf0oo\xb9\xbf\x12@\x1f\xe4\x85\x00\x00\x00\x00IEND\
\xaeB`\x828\x12|\x03'

if __name__ == '__main__':
    test()