import wx

ICON_SIZE = (48,48)

#---vlist
class Dialog(wx.Dialog):
    def __init__(self, *args, **kwds):
        #kwds["style"] = wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER
        super(Dialog,self).__init__(*args,**kwds)
        self.list   = Box(self, -1, style=wx.SUNKEN_BORDER)
        self.status = wx.StaticText(self, -1, "")
        self.cancel = wx.Button(self, wx.ID_CANCEL, _("&Cancel"))
        self.ok     = wx.Button(self, wx.ID_OK, _("&Add"))
        self.ok.SetDefault()
        self._layout()
        
    def _layout(self):
        main_sizer  = wx.BoxSizer(wx.VERTICAL)
        btn_sizer   = wx.BoxSizer(wx.HORIZONTAL)
        main_sizer.Add(self.list, 1, wx.ALL|wx.EXPAND, 4)
        btn_sizer.Add(self.status, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
        btn_sizer.Add(self.cancel, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
        btn_sizer.Add(self.ok, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 4)
        main_sizer.Add(btn_sizer, 0, wx.EXPAND, 0)
        self.SetSizer(main_sizer)
        self.Layout()
        
    def Clear(self):
        self.list.Clear()
        
    def GetIconSize(self):
        return self.list.GetIconSize()
        
    def GetItem(self, n):
        """Needs to be overwritten."""
        return ('label %d'%n, 'summary %d'%n, 
            wx.ArtProvider_GetBitmap(wx.ART_INFORMATION,wx.ART_OTHER,
                self.list.GetIconSize()))
                
    def GetSelection(self):
        return self.list.GetSelection()
    
    def GetSelectedItem(self):
        return self.GetItem(self.GetSelection())
        
    def GetStatusText(self):
        return self.status.GetLabel()
    
    def GetStringSelection(self):
        return self.GetSelectedItem()[0]
        
    def SetIconSize(self,size):
        self.list.SetIconSize(size)
        
    def SetItemCount(self,n):
        self.list.SetItemCount(n)
        
    def SetStatusText(self,text):
        self.status.SetLabel(text)

    def SetTheme(self,theme='default'):
        self.list.SetTheme(theme)
        
    def SetVerticalGradient(self,bool=True):
        self.list.SetVerticalGradient(bool)

class Box(wx.VListBox):
    """Default Icon Size is ICON_SIZE (48)."""
    def __init__(self,*args,**kwds):
        super(Box,self).__init__(*args,**kwds)
        self.SetIconSize(ICON_SIZE)
        self.SetVerticalGradient()
        if wx.Platform=='__WXGTK__':
            self.SetTheme('default')
        else:
            self.SetTheme('light_blue')
        wx.CallAfter(self.SetSelection,0)
        
    def SetTheme(self,theme='default'):
        self._theme = theme
        if theme == 'light_blue': 
            #mozilla like
            self._color_from    = wx.Colour(180,197,214)
            self._color_to      = wx.Colour(217,226,234)
        else:
            self._color_from    = wx.SystemSettings_GetColour(
                                        wx.SYS_COLOUR_MENUHILIGHT)
            self._color_to      = wx.SystemSettings_GetColour(
                                        wx.SYS_COLOUR_HIGHLIGHT)
        
    def SetIconSize(self,icon_size=(48,48),units=5):#31.0):
        """All vertical spacing is calculated by the icon size.
        The higher the units, the less space in between label and summary and
        the more space between the label/summary and the separators."""
        self._icon_size = i_x, i_y = icon_size
        self._row_height= int(1.5*i_y)
        #vertical spacing
        line_height     = self.GetTextExtent('H')[0]
        first_line_unit = int(units/2)
        unit_dy         = (self._row_height-2*line_height)/units
        #positions
        self.icon_x     = (self._row_height - i_x)/2
        self.icon_y     = (self._row_height - i_y)/2
        self.text_x     = 2*self.icon_x+i_x
        self.text_y1    = int(round(first_line_unit*unit_dy))
        self.text_y2    = int(round(
                            (units-first_line_unit)*unit_dy+line_height))
        
    def GetIconSize(self):
        return self._icon_size

    def OnDrawSeparator(self,dc, rect, n):
        dc.SetPen(wx.Pen(wx.LIGHT_GREY,style=wx.DOT))
        y   = rect.GetBottom()
        dc.DrawLine(rect.GetLeft(),y,rect.GetRight(),y)
            
    def OnDrawBackground(self,dc,rect,n):
        """ Gradient fill from colour 1 to colour 2 with top to bottom or left to right. """
        if n!= self.GetSelection():
            return

        if rect.height < 1 or rect.width < 1: 
            return 

        size = (self._is_vertical and [rect.height] or [rect.width])[0]
        start = (self._is_vertical and [rect.y] or [rect.x])[0]

        # calculate gradient coefficients
        col2 = self._color_from
        col1 = self._color_to

        rf, gf, bf = 0, 0, 0
        rstep = float((col2.Red() - col1.Red()))/float(size)
        gstep = float((col2.Green() - col1.Green()))/float(size)
        bstep = float((col2.Blue() - col1.Blue()))/float(size)

        for coord in xrange(start, start + size):
         
            currCol = wx.Colour(col1.Red() + rf, col1.Green() + gf, col1.Blue() + bf)
            dc.SetBrush(wx.Brush(currCol, wx.SOLID)) 
            dc.SetPen(wx.Pen(currCol))
            if self._is_vertical:
                dc.DrawLine(rect.x, coord, rect.x + rect.width, coord) 
            else:
                dc.DrawLine(coord, rect.y, coord, rect.y + rect.height)
                
            rf += rstep
            gf += gstep
            bf += bstep 
            
    def OnMeasureItem(self, n):
        return self._row_height
    
    def OnDrawItem(self, dc, rect, n):
        label, summary, bmp  = self.GetParent().GetItem(n)
        #coordinates
        x0, y0  = rect.GetTopLeft()
        x1, y1  = rect.GetBottomRight()
        #bitmap
        dc.DrawBitmap(bmp, x0+self.icon_x, y0+self.icon_y, True) 
        #text
        if self.GetSelection() != n or self._theme == 'light_blue':
            c = self.GetForegroundColour()
        else:
            c = wx.SystemSettings.GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT)
        bold    = self.GetFont()
        bold.SetWeight(wx.FONTWEIGHT_BOLD)
        dc.SetFont(bold)
        dc.SetTextForeground(c)
        dc.DrawText(label, x0+self.text_x,y0+self.text_y1)
        dc.SetFont(self.GetFont())
        dc.DrawText(summary, x0+self.text_x,y0+self.text_y2)
        
    def SetVerticalGradient(self,bool=True):
        self._is_vertical = bool
        
