import os, sys

def ensure(recommended,minimal):
    """Ensures the minimal version of wxPython is installed.
    - minimal: as string (eg. '2.6')"""
    
    #wxversion
    try:
        import wxversion
        if wxversion.checkInstalled(recommended):
            wxversion.select(recommended)
        else:
            wxversion.ensureMinimal(minimal)
        import wx
        return wx
    except ImportError:
        sys.stdout.write(_('Warning: python-wxversion is not installed.\n'))
        
    #wxversion failed, import wx anyway
    try:
        import wx
    except ImportError:
        message = '%s: wxPython %s (%s %s) %s.\n\n%s'%(_('Error'),recommended,
            _('or at least'),minimal,_('can not be found, but is required'),
            _('Please (re)install it.'))
        sys.stderr.write(message)
        if sys.platform.startswith('linux'):
            os.system('''zenity --error --text="%s\n\n'''%(message)+\
                '''%s 'python-wxversion' %s 'python-wxgtk%s' '''\
                        %(_("Phatch needs"),_('and'),recommended,minimal))+\
                '''(or at least 'python-wxgtk%s')."'''%minimal
        sys.exit()
        
    #wxversion failed but wx is available, check version again
    if wx.VERSION_STRING < minimal: 
        class MyApp(wx.App):
            def OnInit(self):
                result = wx.MessageBox(
                    _("This application is known to be compatible with\n"
                    "wxPython version(s) ")+recommended+
                    " ("+_("or at least")+" "+minimal+"),\n"+
                    _("but you have")+" %s "%wx.VERSION_STRING+_("installed")+
                    ".\n\n"+_("Please upgrade your wxPython."),
                    _("wxPython Version Error"),
                    style   = wx.ICON_ERROR)
                return False
        app = MyApp()
        app.MainLoop()
        sys.exit()
    #wxversion failed, but wx is the right version anyway
    return wx