# -*- coding: UTF-8 -*-

# Phatch recommends SPE (http://pythonide.stani.be) for editing python files.

# Always import this (other imports in method Action.init):
from core import models
from core.translation import _t

# Declare constants here (remove this line, for demonstration only)
CHOICES         = [_t('Sorry'),_t('Image')]

#Use any PIL function you need
def foo(image,dpi,horizontal):
    #process image with pil
    #(...)
    return image

class Action(models.Action):
    
    label       = _t('Label')
    author      = 'Author'
    email       = 'info@email.com'
    version     = '0.1'
    tags        = [_t('tag')]
    __doc__     = _t('Description')
    
    def __init__(self):
        fields = models.Fields()
        
        fields[_t('Boolean')]   = models.BooleanField(True)
        fields[_t('String')]    = models.CharField('hello world')
        fields[_t('Choice')]    = models.ChoiceField(CHOICES[0], CHOICES)
        fields[_t('Colour')]    = models.ColourField('#FFFFFF')
        fields[_t('Resolution')]= models.DpiField('<dpi>')
        fields[_t('File')]      = models.FileField('/home/images/logo.jpg')
        fields[_t('Filename')]  = models.FileNameField('<filename>')
        fields[_t('In')]        = models.FilePathField('<folder>') #folder
        fields[_t('Float')]     = models.FloatField(3.14)
        fields[_t('As')]        = models.ImageTypeField('<type>')#png, jpg
        fields[_t('As')]        = models.ImageReadTypeField('<type>')#png, jpg
        fields[_t('As')]        = models.ImageWriteTypeField('<type>')#png, jpg
        fields[_t('Mode')]      = models.ImageModeField('<mode>')#png, jpg
        fields[_t('Resample')]  = models.ImageResampleField(_t('bicubic'))
        fields[_t('Integer')]   = models.IntegerField(-4)
        fields[_t('Integer+')]  = models.PositiveIntegerField(0)
        fields[_t('Integer+0')] = models.PositiveNoneZeroIntegerField(0)
        fields[_t('Horizontal')]= models.PixelField('5%') #accepts %,cm, inch
        fields[_t('Slider')]    = models.SliderField(60,1,100)
        
        super(Action,self).__init__(fields)
        
    def import_modules(self):
        global Image
        import Image
        
    def apply(self,photo,setting,cache):
        #get info (always get this)
        info                    = photo.get_info()
        
        #in case you use PixelField you can get width, height & dpi ...
        
        #... from photo (use new_*)
        width, height   = info['new_size']
        dpi             = info['new_dpi']
        
        #... or from user input (see actions/image_size.py)
        dpi             = self.get_field_value('Resolution',info)
        
        horizontal      = self.get_field_size('Horizontal Offset',info,
                            width,dpi)
        
        #collect parameters
        parameters  = {
            'dpi'           : dpi,
            'horizontal'    : horizontal,
        }

        #manipulate layer (always return)
        return self.apply_to_current_layer_image(photo,foo,**parameters) 
        
    icon = 'ART_TIP'