#!/usr/bin/env python

import glob, os, sys
from distutils.core import setup
from phatch.data import info

#centralised info generates README and AUTHORS
#info.write_readme_credits()

write   = sys.stdout.write
error   = sys.stderr.write
linux   = sys.platform[:5] == 'linux'

if linux:
    #Create an array with all the locale filenames
    i18n_files = []
    for filepath in glob.glob("locale/*/LC_MESSAGES/*.mo"):
        targetpath = os.path.dirname(os.path.join("share/", filepath))
        i18n_files.append((targetpath, [filepath]))
    #wxGlade
    wxglade_files = [('share/phatch/wxGlade/', 
        glob.glob("phatch/pyWx/wxGlade/*.wxg"))]
    #images & icons
    image_files = [
    
        ('share/phatch/', ["images/phatch-logo.png"]),
        ('share/phatch/pixmaps', glob.glob("images/*.png")),
        
        ('share/applications', ['linux/phatch.desktop']),
        ('share/icons/hicolor/48x48/apps', ['images/phatch-icon.png']),
        ('share/pixmaps', ['images/phatch-icon.png']),
    
    ]

    #include images 
    data_files  = i18n_files + wxglade_files + image_files
else:
    #todo: other platforms
    data_files  = []


dist =  setup(
            packages        = ['phatch','phatch.actions','phatch.console',
                               'phatch.core','phatch.core.lib',
                               'phatch.data','phatch.pyWx','phatch.pyWx.lib',
                               'phatch.pyWx.wxGlade','phatch.templates'],
            scripts         = ['bin/phatch'],
            data_files      = data_files,
            **info.SETUP
        )

if linux:
    # Update the mime types
    if os.geteuid() == 0 and dist != None:
        import subprocess

        #update the .desktop file database
        try:
            subprocess.call(["update-desktop-database"])
            write('Updating the .desktop file database.\n')
        except:
            error('Fail to update the .desktop file database.\n')

write(  "\nInstallation finished! You can now run Phatch by typing 'phatch' \n"
        "or through your applications menu.\n")